/*{ em357_gpio.c ***********************************************
 *
 * Written by Gregor Rebel 2010-2012
 *
 * Low Level Driver for General Purpose Input Output (GPIO) on EM357/EM357 architecture.
 * 
}*/

#include "em357_gpio.h"

/* GPIOMode_TypeDef em357_map_Type(ttc_gpio_mode_e Type) {
    switch (Type) { // find best matching type for em357-architecture
      case tgm_analog_in:                     return GPIO_Mode_AIN;
      case tgm_input_floating:                return GPIO_Mode_IN_FLOATING;
      case tgm_input_pull_down:               return GPIO_Mode_IPD;
      case tgm_input_pull_up:                 return GPIO_Mode_IPU;
      case tgm_output_open_drain:             return GPIO_Mode_Out_OD;
      case tgm_output_push_pull:              return GPIO_Mode_Out_PP;
      case tgm_alternate_function_push_pull:  return GPIO_Mode_AF_PP;
      case tgm_alternate_function_open_drain: return GPIO_Mode_AF_OD;
        
      default: Assert(0, ec_InvalidArgument);
    }
    
    return 0;
} */
GPIOSpeed_TypeDef em357_map_Speed(ttc_gpio_speed_e Speed) {
    return 0; // em357 does not support different GPIO speeds
}

inline void em357_loadPort(em357_Port_t* Port, GPIO_TypeDef* GPIOx, u32_t Pin) {
    Port->GPIOx=GPIOx;
    Port->Pin  =Pin;
}

void em357_initPort2(em357_Port_t* Port, GPIOMode_TypeDef Type, GPIOSpeed_TypeDef Speed) { 

    switch ( (u32_t) Port->GPIOx) { // enable peripheral clock to this port
      case (u32_t) GPIOA: RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA, ENABLE); break;
      case (u32_t) GPIOB: RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOB, ENABLE); break;
      case (u32_t) GPIOC: RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOC, ENABLE); break;
      case (u32_t) GPIOD: RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOD, ENABLE); break;
      case (u32_t) GPIOE: RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOE, ENABLE); break;
      case (u32_t) GPIOF: RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOF, ENABLE); break;
      default: Assert(0, ec_InvalidArgument); break; // ERROR: unknown GPIO given!
    }
    
    GPIO_InitTypeDef GPIO_InitStructure;
    GPIO_InitStructure.GPIO_Mode =Type;
    GPIO_InitStructure.GPIO_Pin  =1 << Port->Pin;
    GPIO_InitStructure.GPIO_Speed=Speed;
    GPIO_Init(Port->GPIOx, &GPIO_InitStructure);
    
    switch (Type) {
      case GPIO_Mode_IPU: {
        break;
      }
      case GPIO_Mode_Out_PP: {
        GPIO_WriteBit(Port->GPIOx, 1 << Port->Pin, Bit_SET);
        break;
      }
      case GPIO_Mode_Out_OD: {
        em357_portSet(Port);
        break;
      }
      default: break;
    }
}

inline bool em357_portGet(em357_Port_t* Port) {
    u32_t Data = Port->GPIOx->IDR;
    if ( Data & (1 << Port->Pin) ) return TRUE;
    return FALSE;
}
inline void em357_portSet(em357_Port_t* Port) {
    Port->GPIOx->BSRR |= 1 << Port->Pin;
}
inline void em357_portClr(em357_Port_t* Port) {
    Port->GPIOx->BRR  |= 1 << Port->Pin;
}
