/*{ stm32_usart::.c -------------------------------------------------------------
 
                      The ToolChain
                      
   Universal Synchronous Asynchronous Receiver and Transmitters (stm32_USARTs)
   for ARM CortexM3 microcontrollers.
   Currently tested to comply to STM32F10x types.
   
   written by Gregor Rebel 2012
   
   Note: See ttc_usart.h for description of architecture independent USART implementation.
 
}*/

#include "stm32_usart.h"

USART_t* stm32_USARTs[TTC_AMOUNT_USARTS];
u8_t su_Initialized=0;

//{ Function definitions --------------------------------------------------------

void stm32_usart_reset_all() {
  memset(stm32_USARTs, 0, sizeof(USART_TypeDef*) * TTC_AMOUNT_USARTS);
  su_Initialized=1;
}
ttc_usart_errorcode_e stm32_usart_get_defaults(u8_t USART_Index, ttc_usart_generic_t* USART_Generic) {
    if (!su_Initialized) stm32_usart_reset_all();
#ifndef TTC_NO_ARGUMENT_CHECKS //{
    // assumption: *USART_Generic has been zeroed

    Assert(USART_Index > 0,    ec_InvalidArgument);
    Assert(USART_Generic != NULL, ec_NULL);
    if (USART_Index > TTC_AMOUNT_USARTS) return tue_DeviceNotFound;
#endif //}

    USART_Generic->Flags.All=0;
    USART_Generic->Flags.SendBreaks     =1;
    USART_Generic->Flags.ControlParity  =1;
    USART_Generic->Flags.Receive        =1;
    USART_Generic->Flags.Transmit       =1;
    USART_Generic->Flags.TransmitParity =1;                                     

    /*{ remaining bits 
    USART_Generic->Flags.LIN            =0;
    USART_Generic->Flags.IrDA           =0;
    USART_Generic->Flags.IrDALowPower   =0;
    USART_Generic->Flags.SmartCardNACK  =0;
    USART_Generic->Flags.RxDMA          =0;
    USART_Generic->Flags.TxDMA          =0;
    USART_Generic->Flags.Rts            =0;
    USART_Generic->Flags.IrqOnCts       =0;
    USART_Generic->Flags.IrqOnError     =0;
    USART_Generic->Flags.IrqOnIdle      =0;
    USART_Generic->Flags.IrqOnTxComplete=0;
    USART_Generic->Flags.IrqOnParityError= 0;
    USART_Generic->Flags.ParityEven     =0;
    USART_Generic->Flags.ParityOdd      =0;

    USART_Generic->Flags.RtsCts         =0;
    USART_Generic->Flags.SingleWire     =0;
    USART_Generic->Flags.SmartCard      =0;
    USART_Generic->Flags.Synchronous    =0;
    }*/

    USART_Generic->WordLength     =8;
    USART_Generic->HalfStopBits   =2;
    USART_Generic->Layout         =0;
    USART_Generic->BaudRate       =115200;

    return tue_OK;
}
ttc_usart_errorcode_e stm32_usart_get_features(u8_t USART_Index, ttc_usart_generic_t* USART_Generic) {
    if (!su_Initialized) stm32_usart_reset_all();
#ifndef TTC_NO_ARGUMENT_CHECKS //{
    // assumption: *USART_Generic has been zeroed

    Assert(USART_Index > 0,    ec_InvalidArgument);
    Assert(USART_Generic != NULL, ec_NULL);
    if (USART_Index > TTC_AMOUNT_USARTS) return tue_DeviceNotFound;
#endif //}

    USART_Generic->Flags.All=0;
    USART_Generic->Flags.Receive        =1;
    USART_Generic->Flags.Transmit       =1;
    USART_Generic->Flags.TransmitParity =1;
    USART_Generic->Flags.SendBreaks     =1;
    USART_Generic->Flags.ParityEven     =1;
    USART_Generic->Flags.ParityOdd      =1;

    /*{ remaining bits
    USART_Generic->Flags.ControlParity  =0; // (ToDo: implement)
    USART_Generic->Flags.RtsCts         =0; // (ToDo: implement)
    USART_Generic->Flags.SingleWire     =0; // (ToDo: implement)
    USART_Generic->Flags.SmartCard      =0; // (ToDo: implement)
    USART_Generic->Flags.Synchronous    =0; // (ToDo: implement)
    USART_Generic->Flags.LIN            =0; // (ToDo: implement)
    USART_Generic->Flags.IrDA           =0; // (ToDo: implement)
    USART_Generic->Flags.IrDALowPower   =0; // (ToDo: implement)
    USART_Generic->Flags.SmartCardNACK  =0; // (ToDo: implement)
    USART_Generic->Flags.RxDMA          =0; // (ToDo: implement)
    USART_Generic->Flags.TxDMA          =0; // (ToDo: implement)
    USART_Generic->Flags.Rts            =0; // (ToDo: implement)
    USART_Generic->Flags.IrqOnCts       =0; // (ToDo: implement)
    USART_Generic->Flags.IrqOnError     =0; // (ToDo: implement)
    USART_Generic->Flags.IrqOnIdle      =0; // (ToDo: implement)
    USART_Generic->Flags.IrqOnTxComplete=0; // (ToDo: implement)
    USART_Generic->Flags.IrqOnParityError= 0; // (ToDo: implement)
    }*/

    USART_Generic->WordLength      = 9;
    USART_Generic->HalfStopBits    = 4;
    USART_Generic->BaudRate        = 115200;

    USART_TypeDef* USART_Base = NULL;
    if (1) { // find indexed USART on current board
      switch (USART_Index) {                // find corresponding USART as defined by makefile.100_board_*
#ifdef TTC_USART1
        case 1: USART_Base = TTC_USART1; break;
#endif
#ifdef TTC_USART2
        case 2: USART_Base = TTC_USART2; break;
#endif
#ifdef TTC_USART3
        case 3: USART_Base = TTC_USART3; break;
#endif
#ifdef TTC_USART4
        case 4: USART_Base = TTC_UART4; break;
#endif
#ifdef TTC_USART5
        case 5: USART_Base = TTC_UART5; break;
#endif
        default: Assert(0, ec_UNKNOWN); break; // No TTC_USARTn defined! Check your makefile.100_board_* file!
        }
    }

    switch ( (u32_t) USART_Base) { // find amount of available remapping-layouts (-> RM0008 p. 176)
      case (u32_t) USART1:
          USART_Generic->Layout=1;     // USART1 and USART2 provide 1 alternate pin layout
          break;
      case (u32_t) USART2:
          if (UC_PACKAGE_PINS >= 100)
              USART_Generic->Layout=1; // USART1 and USART2 provide 1 alternate pin layout
          else
              USART_Generic->Layout=0; // USART1 and USART2 provide 0 alternate pin layout
          break;
      case (u32_t) USART3:
          if (UC_PACKAGE_PINS >= 100)
              USART_Generic->Layout=2; // USART3 provides 2 alternate pin layout
          else if (UC_PACKAGE_PINS >= 64)
              USART_Generic->Layout=1; // USART3 provides 1 alternate pin layout
          else
              USART_Generic->Layout=0; // USART3 provides 0 alternate pin layout
          break;
      case (u32_t) UART4:
      case (u32_t) UART5:
          // UASART4 and USART5 lack some features (-> ST RM0038 Table 138 on p. 651)
          USART_Generic->Flags.Rts            =0;
          USART_Generic->Flags.ControlParity  =0;
          USART_Generic->Flags.RtsCts         =0;
          USART_Generic->Flags.SmartCard      =0;
          USART_Generic->Flags.SmartCardNACK  =0;
          USART_Generic->Flags.Synchronous    =0;
          USART_Generic->Flags.IrqOnCts       =0;
          USART_Generic->Layout               =0; // no alternate pin layout
          
          break;
      default: Assert(0, ec_UNKNOWN); break; // No TTC_USARTn defined! Check your makefile.100_board_* file!
    }
    
    if (1) { // disable features requiring unconfigured pins
      switch (USART_Index) {
        case 1: {
            #ifndef TTC_USART1_TX
              USART_Generic->Flags.Transmit    = 0;
            #endif
            #ifndef TTC_USART1_RX
              USART_Generic->Flags.Receive     = 0;
            #endif
            #ifndef TTC_USART1_RTS
              USART_Generic->Flags.RtsCts      = 0;
              USART_Generic->Flags.Rts         = 0;
            #endif
            #ifndef TTC_USART1_CTS
              USART_Generic->Flags.RtsCts      = 0;
              USART_Generic->Flags.Cts         = 0;
            #endif
            #ifndef TTC_USART1_CK
              USART_Generic->Flags.Synchronous = 0;
            #endif
            
            break;
        }
        case 2: {
            #ifndef TTC_USART2_TX
              USART_Generic->Flags.Transmit    = 0;
            #endif
            #ifndef TTC_USART2_RX
              USART_Generic->Flags.Receive     = 0;
            #endif
            #ifndef TTC_USART2_RTS
              USART_Generic->Flags.RtsCts      = 0;
              USART_Generic->Flags.Rts         = 0;
            #endif
            #ifndef TTC_USART2_CTS
              USART_Generic->Flags.RtsCts      = 0;
              USART_Generic->Flags.Cts         = 0;
            #endif
            #ifndef TTC_USART2_CK
              USART_Generic->Flags.Synchronous = 0;
            #endif          
            
            break;
        }
        case 3: {
            #ifndef TTC_USART3_TX
              USART_Generic->Flags.Transmit    = 0;
            #endif
            #ifndef TTC_USART3_RX
              USART_Generic->Flags.Receive     = 0;
            #endif
            #ifndef TTC_USART3_RTS
              USART_Generic->Flags.RtsCts      = 0;
              USART_Generic->Flags.Rts         = 0;
            #endif
            #ifndef TTC_USART3_CTS
              USART_Generic->Flags.RtsCts      = 0;
              USART_Generic->Flags.Cts         = 0;
            #endif
            #ifndef TTC_USART3_CK
              USART_Generic->Flags.Synchronous = 0;
            #endif
  
            break;
        }
        case 4: {
            #ifndef TTC_USART4_TX
              USART_Generic->Flags.Transmit    = 0;
            #endif
            #ifndef TTC_USART4_RX
              USART_Generic->Flags.Receive     = 0;
            #endif
            
            // UART4, UART5 only provide TX+RX pins
            USART_Generic->Flags.RtsCts      = 0;
            USART_Generic->Flags.Rts         = 0;
            USART_Generic->Flags.Cts         = 0;
            USART_Generic->Flags.Synchronous = 0;
            
            break;
        }
        case 5: {
            #ifndef TTC_USART5_TX
              USART_Generic->Flags.Transmit    = 0;
            #endif
            #ifndef TTC_USART5_RX
              USART_Generic->Flags.Receive     = 0;
            #endif
            
            // UART4, UART5 only provide TX+RX pins
            USART_Generic->Flags.RtsCts      = 0;
            USART_Generic->Flags.Rts         = 0;
            USART_Generic->Flags.Cts         = 0;
            USART_Generic->Flags.Synchronous = 0;
            
            break;
        }
      default: return tue_DeviceNotFound;
      }
    }
    
    return tue_OK;
}
ttc_usart_errorcode_e stm32_usart_init(u8_t USART_Index, ttc_usart_generic_t* USART_Generic, stm32_usart_architecture_t* USART_Arch) {
    if (!su_Initialized) stm32_usart_reset_all();
#ifndef TTC_NO_ARGUMENT_CHECKS //{
    // assumption: *USART_Arch has been zeroed

    Assert(USART_Generic != NULL, ec_NULL);
    if (USART_Index > TTC_AMOUNT_USARTS)
        return tue_DeviceNotFound;
    if ( (USART_Generic->WordLength   < 8) || (USART_Generic->WordLength   > 9) )
        return tue_InvalidWordSize;
    if ( (USART_Generic->HalfStopBits < 1) || (USART_Generic->HalfStopBits > 4) )
        return tue_InvalidStopBits;
#endif //}

    if (1) { // validate USART_Features
        ttc_usart_generic_t USART_Features;
        ttc_usart_errorcode_e Error=stm32_usart_get_features(USART_Index, &USART_Features);
        if (Error) return Error;
        USART_Generic->Flags.All &= USART_Features.Flags.All; // mask out unavailable flags
        if (USART_Generic->Layout > USART_Features.Layout)
            USART_Generic->Layout=USART_Features.Layout;
    }
    if (1) { // find indexed USART on current board
        switch (USART_Index) {                // find corresponding USART as defined by makefile.100_board_*
#ifdef TTC_USART1
        case 1: USART_Arch->Base=TTC_USART1; break;
#endif
#ifdef TTC_USART2
        case 2: USART_Arch->Base=TTC_USART2; break;
#endif
#ifdef TTC_USART3
        case 3: USART_Arch->Base=TTC_USART3; break;
#endif
#ifdef TTC_USART4
        case 4: USART_Arch->Base=TTC_UART4; break;
#endif
#ifdef TTC_USART5
        case 5: USART_Arch->Base=TTC_UART5; break;
#endif
        default: Assert(0, ec_UNKNOWN); break; // No TTC_USARTn defined! Check your makefile.100_board_* file!
        }
    }
    if (1) { // determine pin layout
 //#if defined(STM32F10X_LD) || defined(STM32F10X_LD_VL) || defined(STM32F10X_MD) || defined(STM32F10X_MD_VL) || defined(STM32F10X_HD) || defined(STM32F10X_HD_VL) || defined(STM32F10X_XL) || defined(STM32F10X_CL)
        stm32_Port_t PortTx;
        switch (USART_Index) { // check if board has defined the TX-pin of USART to init
#ifdef TTC_USART1_TX
        case 1:
            stm32_loadPort(&PortTx, TTC_USART1_TX);
            break;
#endif
#ifdef TTC_USART2_TX
        case 2:
            stm32_loadPort(&PortTx, TTC_USART2_TX);
            break;
#endif
#ifdef TTC_USART3_TX
        case 3:
            stm32_loadPort(&PortTx, TTC_USART3_TX);
            break;
#endif
#ifdef TTC_USART4_TX
        case 4:
            stm32_loadPort(&PortTx, TTC_USART4_TX);
            break;
#endif
#ifdef TTC_USART5_TX
        case 5:
            stm32_loadPort(&PortTx, TTC_USART5_TX);
            break;
#endif
        default: Assert(0, ec_UNKNOWN); break; // No TTC_USARTn defined! Check your makefile.100_board_* file!
        }

        switch ( (u32_t) USART_Arch->Base ) { // determine pin layout
        case (u32_t) USART1: { // USART1
            if (PortTx.GPIOx == GPIOB) USART_Generic->Layout=1;
            else                       USART_Generic->Layout=0;

            if (USART_Generic->Layout) { // alternate pin remapping (-> ST RM0041 STM32F100xx advanced ARM-based 32-bit MCUs p.118)
                USART_Arch->PortTxD.GPIOx=GPIOB;
                USART_Arch->PortTxD.Pin  =6;
                USART_Arch->PortRxD.GPIOx=GPIOB;
                USART_Arch->PortRxD.Pin  =7;
            }
            else {                       // no pin remapping
                USART_Arch->PortTxD.GPIOx=GPIOA;
                USART_Arch->PortTxD.Pin  =9;
                USART_Arch->PortRxD.GPIOx=GPIOA;
                USART_Arch->PortRxD.Pin  =10;
            }

            USART_Arch->PortCTS.GPIOx=GPIOA;
            USART_Arch->PortCTS.Pin  =11;
            USART_Arch->PortRTS.GPIOx=GPIOA;
            USART_Arch->PortRTS.Pin  =12;
            USART_Arch->PortCLK.GPIOx=GPIOA;
            USART_Arch->PortCLK.Pin  =8;
            break;
        }
        case (u32_t) USART2: { // USART2
            if (PortTx.GPIOx == GPIOD) USART_Generic->Layout=1;
            else                       USART_Generic->Layout=0;

            if (USART_Generic->Layout) { // alternate pin remapping (-> ST RM0041 STM32F100xx advanced ARM-based 32-bit MCUs p.118)
                USART_Arch->PortCTS.GPIOx=GPIOD;
                USART_Arch->PortCTS.Pin  =3;
                USART_Arch->PortRTS.GPIOx=GPIOD;
                USART_Arch->PortRTS.Pin  =4;
                USART_Arch->PortTxD.GPIOx=GPIOD;
                USART_Arch->PortTxD.Pin  =5;
                USART_Arch->PortRxD.GPIOx=GPIOD;
                USART_Arch->PortRxD.Pin  =6;
                USART_Arch->PortCLK.GPIOx=GPIOD;
                USART_Arch->PortCLK.Pin  =7;
            }
            else {                       // no pin remapping
                USART_Arch->PortCTS.GPIOx=GPIOA;
                USART_Arch->PortCTS.Pin  =0;
                USART_Arch->PortRTS.GPIOx=GPIOA;
                USART_Arch->PortRTS.Pin  =1;
                USART_Arch->PortTxD.GPIOx=GPIOA;
                USART_Arch->PortTxD.Pin  =2;
                USART_Arch->PortRxD.GPIOx=GPIOA;
                USART_Arch->PortRxD.Pin  =3;
                USART_Arch->PortCLK.GPIOx=GPIOA;
                USART_Arch->PortCLK.Pin  =4;
            }
            break;
        }
        case (u32_t) USART3: { // USART3
            if (PortTx.GPIOx == GPIOB)       USART_Generic->Layout=1; // partial pin remapping
            else if (PortTx.GPIOx == GPIOD)  USART_Generic->Layout=2; // full pin remapping
            else                             USART_Generic->Layout=0; // no pin remapping

            switch (USART_Generic->Layout) { // (-> ST RM0041 STM32F100xx advanced ARM-based 32-bit MCUs p.118)
            case 1: {  // partial pin remapping
                USART_Arch->PortTxD.GPIOx=GPIOC;
                USART_Arch->PortTxD.Pin  =10;
                USART_Arch->PortRxD.GPIOx=GPIOC;
                USART_Arch->PortRxD.Pin  =11;
                USART_Arch->PortCLK.GPIOx=GPIOC;
                USART_Arch->PortCLK.Pin  =12;
                USART_Arch->PortCTS.GPIOx=GPIOB;
                USART_Arch->PortCTS.Pin  =13;
                USART_Arch->PortRTS.GPIOx=GPIOB;
                USART_Arch->PortRTS.Pin  =14;
                break;
            }
            case 2: {  // full pin remapping
                USART_Arch->PortTxD.GPIOx=GPIOD;
                USART_Arch->PortTxD.Pin  =8;
                USART_Arch->PortRxD.GPIOx=GPIOD;
                USART_Arch->PortRxD.Pin  =9;
                USART_Arch->PortCLK.GPIOx=GPIOD;
                USART_Arch->PortCLK.Pin  =10;
                USART_Arch->PortCTS.GPIOx=GPIOD;
                USART_Arch->PortCTS.Pin  =11;
                USART_Arch->PortRTS.GPIOx=GPIOD;
                USART_Arch->PortRTS.Pin  =12;
                break;
            }
            default: { // no pin remapping
                USART_Arch->PortTxD.GPIOx=GPIOB;
                USART_Arch->PortTxD.Pin  =10;
                USART_Arch->PortRxD.GPIOx=GPIOB;
                USART_Arch->PortRxD.Pin  =11;
                USART_Arch->PortCLK.GPIOx=GPIOB;
                USART_Arch->PortCLK.Pin  =12;
                USART_Arch->PortCTS.GPIOx=GPIOB;
                USART_Arch->PortCTS.Pin  =13;
                USART_Arch->PortRTS.GPIOx=GPIOB;
                USART_Arch->PortRTS.Pin  =14;
                break;
            }
            }
            break;
        }
        case (u32_t) UART4:  { // UART4
            USART_Arch->PortTxD.GPIOx=GPIOC;
            USART_Arch->PortTxD.Pin  =10;
            USART_Arch->PortRxD.GPIOx=GPIOC;
            USART_Arch->PortRxD.Pin  =11;
            break;
        }
        case (u32_t) UART5:  { // UART5
            USART_Arch->PortTxD.GPIOx=GPIOC;
            USART_Arch->PortTxD.Pin  =12;
            USART_Arch->PortRxD.GPIOx=GPIOD;
            USART_Arch->PortRxD.Pin  =2;
            break;
        }
        default: { return tue_DeviceNotFound; }
        }
    }
    if (1) { // compare chosen pin layout with configured pins
        //tue_InvalidPinConfig
        stm32_Port_t Port;
        switch (USART_Index) { // check if board has defined the TX-pin of USART to init
          
          case 1: { // check configuration of USART #1
  #ifdef TTC_USART1_TX
              stm32_loadPort(&Port, TTC_USART1_TX);
              Assert(Port.GPIOx == USART_Arch->PortTxD.GPIOx, ec_InvalidArgument); // ERROR: configured GPIO cannot be used in current remapping layout!
              Assert(Port.Pin   == USART_Arch->PortTxD.Pin,   ec_InvalidArgument); // ERROR: configured Pin  cannot be used in current remapping layout!
  #else
              USART_Arch->PortTxD.GPIOx = NULL;
  #endif
  #ifdef TTC_USART1_RX
              stm32_loadPort(&Port, TTC_USART1_RX);
              Assert(Port.GPIOx == USART_Arch->PortRxD.GPIOx, ec_InvalidArgument); // ERROR: configured GPIO cannot be used in current remapping layout!
              Assert(Port.Pin   == USART_Arch->PortRxD.Pin,   ec_InvalidArgument); // ERROR: configured Pin  cannot be used in current remapping layout!
  #else
              USART_Arch->PortRxD.GPIOx = NULL;
  #endif
  #ifdef TTC_USART1_RTS
              stm32_loadPort(&Port, TTC_USART1_RTS);
              Assert(Port.GPIOx == USART_Arch->PortRTS.GPIOx, ec_InvalidArgument); // ERROR: configured GPIO cannot be used in current remapping layout!
              Assert(Port.Pin   == USART_Arch->PortRTS.Pin,   ec_InvalidArgument); // ERROR: configured Pin  cannot be used in current remapping layout!
  #else
              USART_Arch->PortRTS.GPIOx = NULL;
  #endif
  #ifdef TTC_USART1_CTS
              stm32_loadPort(&Port, TTC_USART1_CTS);
              Assert(Port.GPIOx == USART_Arch->PortCTS.GPIOx, ec_InvalidArgument); // ERROR: configured GPIO cannot be used in current remapping layout!
              Assert(Port.Pin   == USART_Arch->PortCTS.Pin,   ec_InvalidArgument); // ERROR: configured Pin  cannot be used in current remapping layout!
  #else
              USART_Arch->PortCTS.GPIOx = NULL;
  #endif
  #ifdef TTC_USART1_CK
              stm32_loadPort(&Port, TTC_USART1_CK);
              Assert(Port.GPIOx == USART_Arch->PortCLK.GPIOx, ec_InvalidArgument); // ERROR: configured GPIO cannot be used in current remapping layout!
              Assert(Port.Pin   == USART_Arch->PortCLK.Pin,   ec_InvalidArgument); // ERROR: configured Pin  cannot be used in current remapping layout!
  #else
              USART_Arch->PortCLK.GPIOx = NULL;
  #endif
              break;
          }
          case 2: { // check configuration of USART #2
  #ifdef TTC_USART2_TX
              stm32_loadPort(&Port, TTC_USART2_TX);
              Assert(Port.GPIOx == USART_Arch->PortTxD.GPIOx, ec_InvalidArgument); // ERROR: configured GPIO cannot be used in current remapping layout!
              Assert(Port.Pin   == USART_Arch->PortTxD.Pin,   ec_InvalidArgument); // ERROR: configured Pin  cannot be used in current remapping layout!
  #else
              USART_Arch->PortTxD.GPIOx = NULL;
  #endif
  #ifdef TTC_USART2_RX
              stm32_loadPort(&Port, TTC_USART2_RX);
              Assert(Port.GPIOx == USART_Arch->PortRxD.GPIOx, ec_InvalidArgument); // ERROR: configured GPIO cannot be used in current remapping layout!
              Assert(Port.Pin   == USART_Arch->PortRxD.Pin,   ec_InvalidArgument); // ERROR: configured Pin  cannot be used in current remapping layout!
  #else
              USART_Arch->PortRxD.GPIOx = NULL;
  #endif
  #ifdef TTC_USART2_RTS
              stm32_loadPort(&Port, TTC_USART2_RTS);
              Assert(Port.GPIOx == USART_Arch->PortRTS.GPIOx, ec_InvalidArgument); // ERROR: configured GPIO cannot be used in current remapping layout!
              Assert(Port.Pin   == USART_Arch->PortRTS.Pin,   ec_InvalidArgument); // ERROR: configured Pin  cannot be used in current remapping layout!
  #else
              USART_Arch->PortRTS.GPIOx = NULL;
  #endif
  #ifdef TTC_USART2_CTS
              stm32_loadPort(&Port, TTC_USART2_CTS);
              Assert(Port.GPIOx == USART_Arch->PortCTS.GPIOx, ec_InvalidArgument); // ERROR: configured GPIO cannot be used in current remapping layout!
              Assert(Port.Pin   == USART_Arch->PortCTS.Pin,   ec_InvalidArgument); // ERROR: configured Pin  cannot be used in current remapping layout!
  #else
              USART_Arch->PortCTS.GPIOx = NULL;
  #endif
  #ifdef TTC_USART2_CK
              stm32_loadPort(&Port, TTC_USART2_CK);
              Assert(Port.GPIOx == USART_Arch->PortCLK.GPIOx, ec_InvalidArgument); // ERROR: configured GPIO cannot be used in current remapping layout!
              Assert(Port.Pin   == USART_Arch->PortCLK.Pin,   ec_InvalidArgument); // ERROR: configured Pin  cannot be used in current remapping layout!
  #else
              USART_Arch->PortCLK.GPIOx = NULL;
  #endif
              break;
          }
          case 3: { // check configuration of USART #3
  #ifdef TTC_USART3_TX
              stm32_loadPort(&Port, TTC_USART3_TX);
              Assert(#.GPIOx == USART_Arch->PortTxD.GPIOx, ec_InvalidArgument); // ERROR: configured GPIO cannot be used in current remapping layout!
              Assert(Port.Pin   == USART_Arch->PortTxD.Pin,   ec_InvalidArgument); // ERROR: configured Pin  cannot be used in current remapping layout!
  #else
              USART_Arch->PortTxD.GPIOx = NULL;
  #endif
  #ifdef TTC_USART3_RX
              stm32_loadPort(&Port, TTC_USART3_RX);
              Assert(Port.GPIOx == USART_Arch->PortRxD.GPIOx, ec_InvalidArgument); // ERROR: configured GPIO cannot be used in current remapping layout!
              Assert(Port.Pin   == USART_Arch->PortRxD.Pin,   ec_InvalidArgument); // ERROR: configured Pin  cannot be used in current remapping layout!
  #else
              USART_Arch->PortRxD.GPIOx = NULL;
  #endif
  #ifdef TTC_USART3_RTS
              stm32_loadPort(&Port, TTC_USART3_RTS);
              Assert(Port.GPIOx == USART_Arch->PortRTS.GPIOx, ec_InvalidArgument); // ERROR: configured GPIO cannot be used in current remapping layout!
              Assert(Port.Pin   == USART_Arch->PortRTS.Pin,   ec_InvalidArgument); // ERROR: configured Pin  cannot be used in current remapping layout!
  #else
              USART_Arch->PortRTS.GPIOx = NULL;
  #endif
  #ifdef TTC_USART3_CTS
              stm32_loadPort(&Port, TTC_USART3_CTS);
              Assert(Port.GPIOx == USART_Arch->PortCTS.GPIOx, ec_InvalidArgument); // ERROR: configured GPIO cannot be used in current remapping layout!
              Assert(Port.Pin   == USART_Arch->PortCTS.Pin,   ec_InvalidArgument); // ERROR: configured Pin  cannot be used in current remapping layout!
  #else
              USART_Arch->PortCTS.GPIOx = NULL;
  #endif
  #ifdef TTC_USART3_CK
              stm32_loadPort(&Port, TTC_USART3_CK);
              Assert(Port.GPIOx == USART_Arch->PortCLK.GPIOx, ec_InvalidArgument); // ERROR: configured GPIO cannot be used in current remapping layout!
              Assert(Port.Pin   == USART_Arch->PortCLK.Pin,   ec_InvalidArgument); // ERROR: configured Pin  cannot be used in current remapping layout!
  #else
              USART_Arch->PortCLK.GPIOx = NULL;
  #endif
              break;
          }
          case 4: { // check configuration of USART #4
  #ifdef TTC_USART4_TX
              stm32_loadPort(&Port, TTC_USART4_TX);
              Assert(Port.GPIOx == USART_Arch->PortTxD.GPIOx, ec_InvalidArgument); // ERROR: configured GPIO cannot be used in current remapping layout!
              Assert(Port.Pin   == USART_Arch->PortTxD.Pin,   ec_InvalidArgument); // ERROR: configured Pin  cannot be used in current remapping layout!
  #else
              USART_Arch->PortTxD.GPIOx = NULL;
  #endif
  #ifdef TTC_USART4_RX
              stm32_loadPort(&Port, TTC_USART4_RX);
              Assert(Port.GPIOx == USART_Arch->PortRxD.GPIOx, ec_InvalidArgument); // ERROR: configured GPIO cannot be used in current remapping layout!
              Assert(Port.Pin   == USART_Arch->PortRxD.Pin,   ec_InvalidArgument); // ERROR: configured Pin  cannot be used in current remapping layout!
  #else
              USART_Arch->PortRxD.GPIOx = NULL;
  #endif
              break;
          }
          case 5: { // check configuration of USART #5
  #ifdef TTC_USART5_TX
              stm32_loadPort(&Port, TTC_USART5_TX);
              Assert(Port.GPIOx == USART_Arch->PortTxD.GPIOx, ec_InvalidArgument); // ERROR: configured GPIO cannot be used in current remapping layout!
              Assert(Port.Pin   == USART_Arch->PortTxD.Pin,   ec_InvalidArgument); // ERROR: configured Pin  cannot be used in current remapping layout!
  #else
              USART_Arch->PortTxD.GPIOx = NULL;
  #endif
  #ifdef TTC_USART5_RX
              stm32_loadPort(&Port, TTC_USART5_RX);
              Assert(Port.GPIOx == USART_Arch->PortRxD.GPIOx, ec_InvalidArgument); // ERROR: configured GPIO cannot be used in current remapping layout!
              Assert(Port.Pin   == USART_Arch->PortRxD.Pin,   ec_InvalidArgument); // ERROR: configured Pin  cannot be used in current remapping layout!
  #else
              USART_Arch->PortRxD.GPIOx = NULL;
  #endif
              break;
          }
        default: Assert(0, ec_UNKNOWN); break; // No TTC_USARTn defined! Check your makefile.100_board_* file!
            
        }
    }
    if (1) { // activate clocks
        switch ( (u32_t) USART_Arch->Base ) {
        case (u32_t) USART1: {
            if (USART_Generic->Layout) { // GPIOB
                RCC_APB2PeriphClockCmd(RCC_APB2Periph_USART1 | RCC_APB2Periph_GPIOB | RCC_APB2Periph_AFIO, ENABLE);
            }
            else {                       // GPIOA
                RCC_APB2PeriphClockCmd(RCC_APB2Periph_USART1 | RCC_APB2Periph_GPIOA | RCC_APB2Periph_AFIO, ENABLE);
            }
            break;
        }
        case (u32_t) USART2: {
            if (USART_Generic->Layout) { // GPIOD
                RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART2 , ENABLE);
                RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOD | RCC_APB2Periph_AFIO, ENABLE);
            }
            else {                       // GPIOA
                RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART2 , ENABLE);
                RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA | RCC_APB2Periph_AFIO, ENABLE);
            }
            break;
        }
        case (u32_t) USART3: {
            switch (USART_Generic->Layout) {
            case 1: // GPPIOB, GPIOC
                RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART3, ENABLE);
                RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOB | RCC_APB2Periph_GPIOC | RCC_APB2Periph_AFIO, ENABLE);
                break;
            case 2: // GPPIOD
                RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART3, ENABLE);
                RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOD | RCC_APB2Periph_AFIO, ENABLE);
                break;
            default: // GPPIOB
                RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART3, ENABLE);
                RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOB | RCC_APB2Periph_AFIO, ENABLE);
                break;
            }
            break;
        }
        case (u32_t) UART4:  { 
            RCC_APB1PeriphClockCmd(RCC_APB1Periph_UART4 , ENABLE);
            RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOD | RCC_APB2Periph_AFIO, ENABLE);
            break;
        }
        case (u32_t) UART5:  { 
            RCC_APB1PeriphClockCmd(RCC_APB1Periph_UART5 , ENABLE);
            RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOC | RCC_APB2Periph_GPIOD | RCC_APB2Periph_AFIO, ENABLE);
            break;
        }
        default: Assert(0, ec_UNKNOWN); break; // should never occur!
        }
    }
    USART_DeInit(USART_Arch->Base);
    if (1) { // configure GPIOs
        if (USART_Arch->PortTxD.GPIOx)  stm32_initPort2(&(USART_Arch->PortTxD), GPIO_Mode_AF_PP,       GPIO_Speed_50MHz);
        if (USART_Arch->PortRxD.GPIOx)  stm32_initPort2(&(USART_Arch->PortRxD), GPIO_Mode_IN_FLOATING, GPIO_Speed_50MHz);
        if (USART_Arch->PortRTS.GPIOx)  stm32_initPort2(&(USART_Arch->PortRTS), GPIO_Mode_AF_PP,       GPIO_Speed_50MHz);
        if (USART_Arch->PortCLK.GPIOx)  stm32_initPort2(&(USART_Arch->PortCLK), GPIO_Mode_AF_PP,       GPIO_Speed_50MHz);
        if (USART_Arch->PortCTS.GPIOx)  stm32_initPort2(&(USART_Arch->PortCTS), GPIO_Mode_IN_FLOATING, GPIO_Speed_50MHz);
    }
    if (1) { // apply remap layout

        switch ( (u32_t) USART_Arch->Base ) {

        case (u32_t) USART1: { // USART1
            if (USART_Generic->Layout)
                GPIO_PinRemapConfig(GPIO_Remap_USART1, ENABLE);
            else
                GPIO_PinRemapConfig(GPIO_Remap_USART1, DISABLE);

            break;
        }
        case (u32_t) USART2: { // USART2
            if (USART_Generic->Layout)
                GPIO_PinRemapConfig(GPIO_Remap_USART2, ENABLE);
            else
                GPIO_PinRemapConfig(GPIO_Remap_USART2, DISABLE);
            break;
        }
        case (u32_t) USART3: { // USART3
            switch (USART_Generic->Layout) { // (-> ST RM0041 STM32F100xx advanced ARM-based 32-bit MCUs p.118)
            case 1:
                GPIO_PinRemapConfig(GPIO_PartialRemap_USART3, ENABLE);
                break;
            case 2:
                GPIO_PinRemapConfig(GPIO_FullRemap_USART3, ENABLE);
                break;
            default:  // no pin remapping
                GPIO_PinRemapConfig(GPIO_PartialRemap_USART3 | GPIO_FullRemap_USART3, DISABLE);
                break;
            }
            break;
        }

        default: { break; }
        }

    }
    if (1) { // init USART

        USART_InitTypeDef USART_MyInit;
        USART_MyInit.USART_BaudRate=USART_Generic->BaudRate;

        USART_MyInit.USART_HardwareFlowControl= USART_HardwareFlowControl_None;
        if (USART_Generic->Flags.Rts)     USART_MyInit.USART_HardwareFlowControl= USART_HardwareFlowControl_RTS;
        if (USART_Generic->Flags.Cts)     USART_MyInit.USART_HardwareFlowControl= USART_HardwareFlowControl_CTS;
        if (USART_Generic->Flags.RtsCts)  USART_MyInit.USART_HardwareFlowControl= USART_HardwareFlowControl_RTS_CTS;

        USART_MyInit.USART_Mode= 0;
        if (USART_Generic->Flags.Transmit) USART_MyInit.USART_Mode |= USART_Mode_Tx;
        if (USART_Generic->Flags.Receive)  USART_MyInit.USART_Mode |= USART_Mode_Rx;

        USART_MyInit.USART_Parity=USART_Parity_No;
        if (USART_Generic->Flags.ParityEven) USART_MyInit.USART_Parity=USART_Parity_Even;
        if (USART_Generic->Flags.ParityOdd)  USART_MyInit.USART_Parity=USART_Parity_Odd;

        switch (USART_Generic->HalfStopBits) {
        case 1: USART_MyInit.USART_StopBits  =USART_StopBits_0_5; break;
        case 3: USART_MyInit.USART_StopBits  =USART_StopBits_1_5; break;
        case 4: USART_MyInit.USART_StopBits  =USART_StopBits_2;   break;
        default:
        case 2: USART_MyInit.USART_StopBits  =USART_StopBits_1;   break;
        }

        switch (USART_Generic->WordLength) {
        case 9: USART_MyInit.USART_WordLength=USART_WordLength_9b; break;

        default:
        case 8: USART_MyInit.USART_WordLength=USART_WordLength_8b; break;
        }

        // translate USART_MyInit back to USART_Generic
        switch (USART_MyInit.USART_HardwareFlowControl) {
        case USART_HardwareFlowControl_RTS:
            USART_Generic->Flags.RtsCts=0;
            USART_Generic->Flags.Rts   =1;
            USART_Generic->Flags.Cts   =0;
            break;
        case USART_HardwareFlowControl_CTS:
            USART_Generic->Flags.RtsCts=0;
            USART_Generic->Flags.Rts   =0;
            USART_Generic->Flags.Cts   =1;
            break;
        case USART_HardwareFlowControl_RTS_CTS:
            USART_Generic->Flags.RtsCts=1;
            USART_Generic->Flags.Rts   =1;
            USART_Generic->Flags.Cts   =1;
            break;
        default:
            USART_Generic->Flags.RtsCts=0;
            USART_Generic->Flags.Rts   =0;
            USART_Generic->Flags.Cts   =0;
            break;
        }
        switch (USART_MyInit.USART_Parity) {
        case USART_Parity_Even:
            USART_Generic->Flags.ParityEven=1;
            USART_Generic->Flags.ParityOdd =0;
            break;
        case USART_Parity_Odd:
            USART_Generic->Flags.ParityEven=0;
            USART_Generic->Flags.ParityOdd =1;
            break;
        default:
            USART_Generic->Flags.ParityEven=0;
            USART_Generic->Flags.ParityOdd =0;
            break;
        }
        switch (USART_MyInit.USART_StopBits) {
        case USART_StopBits_0_5:  USART_Generic->HalfStopBits=1; break;
        case USART_StopBits_1_5:  USART_Generic->HalfStopBits=3; break;
        case USART_StopBits_2:    USART_Generic->HalfStopBits=4; break;
        default:
        case USART_StopBits_1:    USART_Generic->HalfStopBits=2; break;
        }
        switch (USART_MyInit.USART_WordLength) {
        case USART_WordLength_9b:
            USART_Generic->WordLength=9; break;
        default:
            USART_Generic->WordLength=8; break;
        }

        if (USART_MyInit.USART_Mode & USART_Mode_Tx) USART_Generic->Flags.Transmit=1;
        if (USART_MyInit.USART_Mode & USART_Mode_Rx) USART_Generic->Flags.Receive =1;

        USART_Init(USART_Arch->Base, &USART_MyInit);

    }
    if (0) { // setup interrupts
        USART_ITConfig(USART_Arch->Base, USART_IT_RXNE, ENABLE);

        /* Enable and set USARTn Interrupt to the lowest priority */
        NVIC_InitTypeDef NVIC_InitStructure;
        switch ( (u32_t) USART_Arch->Base ) {

        case (u32_t) USART1: NVIC_InitStructure.NVIC_IRQChannel=USART1_IRQn; break;
        case (u32_t) USART2: NVIC_InitStructure.NVIC_IRQChannel=USART2_IRQn; break;
        case (u32_t) USART3: NVIC_InitStructure.NVIC_IRQChannel=USART3_IRQn; break;

        default: { break; }
        }

        NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority=0x0F;
        NVIC_InitStructure.NVIC_IRQChannelSubPriority=0x0F;
        NVIC_InitStructure.NVIC_IRQChannelCmd=ENABLE;
        NVIC_Init(&NVIC_InitStructure);
    }
    
    USART_Cmd(USART_Arch->Base, ENABLE);
    stm32_USARTs[USART_Index]=(USART_t*) USART_Arch->Base;
    return tue_OK;
}
ttc_usart_errorcode_e stm32_usart_send_raw(u8_t USART_Index, const char* Buffer, u16_t Amount) {
  USART_t* MyUSART=stm32_USARTs[USART_Index];
  Assert(MyUSART != NULL, ec_NULL);

  while (Amount-- > 0) {
    _stm32_usart_send_word(MyUSART, (u16_t) *Buffer++);
  }
  return tue_OK;
}
ttc_usart_errorcode_e stm32_usart_send_string(u8_t USART_Index, const char* Buffer, u16_t MaxLength) {
    USART_t* MyUSART=stm32_USARTs[USART_Index];
    Assert(MyUSART != NULL, ec_NULL);

    char C=0;
    while ( (MaxLength-- > 0) && ((C=*Buffer++) != 0)  ) {
      _stm32_usart_send_word(MyUSART, (u16_t) C);
    }
    return tue_OK;
}
ttc_usart_errorcode_e stm32_usart_send_word(u8_t USART_Index, const u16_t Word) {
    USART_t* MyUSART=stm32_USARTs[USART_Index];
    Assert(MyUSART != NULL, ec_NULL);

    _stm32_usart_send_word(MyUSART, Word);
    return tue_OK;
}
ttc_usart_errorcode_e stm32_usart_read_byte(u8_t USART_Index, char* Byte, u32_t TimeOut) {
    USART_t* MyUSART=stm32_USARTs[USART_Index];
    Assert(MyUSART != NULL, ec_NULL);

    if ( _stm32_usart_wait_for_RXNE(MyUSART, TimeOut) ) return tue_TimeOut;

    *Byte=(char) 0xff & USART_ReceiveData( (USART_TypeDef*) MyUSART );
    // MyUSART->USART_SR.RXNE=0;

    return tue_OK;
}
ttc_usart_errorcode_e stm32_usart_read_word(u8_t USART_Index, u16_t* Word, u32_t TimeOut) {
    USART_t* MyUSART=stm32_USARTs[USART_Index];
    Assert(MyUSART != NULL, ec_NULL);

    if ( _stm32_usart_wait_for_RXNE(MyUSART, TimeOut) ) return tue_TimeOut;

    *Word=USART_ReceiveData( (USART_TypeDef*) MyUSART  );
    // MyUSART->USART_SR.RXNE=0;

    return tue_OK;
}

//} Function definitions
//{ private functions (ideally) -------------------------------------------------

                 void _stm32_usart_send_word(USART_t* MyUSART, const u16_t Word) {
    Assert(MyUSART != NULL, ec_NULL);

    while (USART_GetFlagStatus( (USART_TypeDef*) MyUSART , USART_FLAG_TXE) == RESET) uSleep(10);
    USART_SendData( (USART_TypeDef*) MyUSART , Word);
}
ttc_usart_errorcode_e _stm32_usart_wait_for_RXNE(USART_t* MyUSART, u32_t TimeOut) {
    if (TimeOut > 0) { // wait until word received or timeout
        while ( (TimeOut > 0) && (MyUSART->USART_SR.RXNE == 0) ) {
            if (TimeOut > 9) TimeOut -= 10;
            else             TimeOut=0;
            uSleep(10);
        }
        if (TimeOut == 0)  return tue_TimeOut;
    }
    else {             // wait until word received (no timeout)
        while (MyUSART->USART_SR.RXNE == 0)
            uSleep(10);
    }

    return tue_OK;
}

//} private functions
