/*{ stm32_basic.h ***********************************************
 *
 * TheToolChain.com
 *
 * Written by Gregor Rebel 2010-2012
 *
 * Basic datatypes, enums and functions
 * 
}*/

#ifndef TTC_BASIC_H
#define TTC_BASIC_H


typedef unsigned char        u8_t;
typedef unsigned short int  u16_t;
typedef unsigned long       u32_t;
typedef signed char          s8_t;
typedef signed short int    s16_t;
typedef signed int          s32_t;
typedef enum { FALSE=0, TRUE } BOOL;
typedef BOOL bool;

#ifndef NULL
#define NULL 0
#endif

typedef enum ErrorCode_E {
  ec_None,                           // 0
  ec_Malloc,                         // 1
  ec_NULL,
  ec_InvalidArgument,
  ec_NotImplemented,
  ec_UNKNOWN
} ErrorCode_e;

/*
#if TARGET_DATA_WIDTH==32
  #define strnlen strnlen__32
#endif
#if TARGET_DATA_WIDTH==16
  #define strnlen strnlen__32
#endif
#ifndef strnlen
  #define strnlen strnlen__8
#endif
*/

//{ Function prototypes **************************************************

/* Will block endlessly if Condition is false. Use it to check arguments and system states.
 * @param Condition  =0: Assert() will block in an endless while-loop; !=0: Assert() returns immediately
 * @param ErrorCode  developers hint why condition has failed
 * @return value of Condition
 */
bool Assert(volatile bool Condition, volatile ErrorCode_e ErrorCode);

/* Calculate length of given string as amount of characters until first zero byte or MaxLen reached.
 * This function can be used for strings of size up to 255 bytes.
 * Note: You may use the architecture aware macro strnlen() instead!
 *
 * @param String  points to buffer storing string to be read
 * @param MaxLen  safety value used to avoid running out of allocated memory in case of missing zero byte
 * @return length of string <= MaxLen
 */
u8_t strnlen__8(const char* String, u8_t MaxLen);

/* Calculate length of given string as amount of characters until first zero byte or MaxLen reached.
 * This function can be used for strings of size up to 65535 bytes.
 * Note: You may use the architecture aware macro strnlen() instead!
 *
 * @param String  points to buffer storing string to be read
 * @param MaxLen  safety value used to avoid running out of allocated memory in case of missing zero byte
 * @return length of string <= MaxLen
 */
u16_t strnlen__16(const char* String, u16_t MaxLen);

/* Calculate length of given string as amount of characters until first zero byte or MaxLen reached.
 * This function can be used for strings of size up to 4 GB (theoretically)
 * Note: You may use the architecture aware macro strnlen() instead!
 *
 * @param String  points to buffer storing string to be read
 * @param MaxLen  safety value used to avoid running out of allocated memory in case of missing zero byte
 * @return length of string <= MaxLen
 */
u32_t strnlen__32(const char* String, u32_t MaxLen);

//}Function prototypes
#endif
