#ifndef TTC_GPIO_H
#define TTC_GPIO_H

/*{ ttc_gpio.h ***********************************************
 
 * Written by Gregor Rebel 2010-2012
 *
 * Architecture independent support for General Purpose Input Output (GPIO).
 * 
 * Currently implemented architectures: stm32
 *
 * Example usage:
 
 Makefile
    COMPILE_OPTS += -DTTC_LED1=PIN_PA8
 
 Source
    #include "ttc_gpio.h"
 
    ttc_Port_t LED;
    ttc_initPort(&LED, TTC_LED1, tgm_output_push_pull);
    while (1) {
      ttc_portSet(&Port);
      ttc_portClr(&Port);
      }
}*/
//{ includes

#include "ttc_basic.h"
#include "ttc_gpio_types.h"
#ifdef TARGET_ARCHITECTURE_STM32
  #include "stm32_gpio.h"
#endif

//}includes
//{ Prototypes

/* initializes single port bit for use as input/ output with maximum speed
 * @param GPIOx  GPIO bank to use (-> ttc_gpio_types.h:ttc_GPIO_BANK)
 * @param Pin    0..15
 * @param Type   mode to use (-> ttc_gpio_types.h:ttc_gpio_mode_e) 
 */
void ttc_initPort(ttc_Port_t* Port, ttc_GPIO_BANK GPIOx, u32_t Pin, ttc_gpio_mode_e Type);

/* initializes single port bit for use as input/ output with maximum speed
 * @param GPIOx  GPIO bank to use (-> ttc_gpio_types.h:ttc_GPIO_BANK)
 * @param Pin    0..15
 * @param Type   mode to use (-> ttc_gpio_types.h:ttc_gpio_mode_e) 
 * @param Speed  speed to use (-> ttc_gpio_types.h:ttc_gpio_speed_e) 
 */
void ttc_initPort2(ttc_Port_t* Port, ttc_GPIO_BANK GPIOx, u32_t Pin, ttc_gpio_mode_e Type, ttc_gpio_speed_e Speed);

/* allows to load struct from list type definition
 *
 * @param Port   points to structure to be filled with given data
 * @param GPIOx  one of GPIOA, GPIOB, GPIOC, GPIOD, GPIOE, GPIOF
 * @param Pin    0..15
 * @return  filled out struct *Port
 *
 * Example:
 * Makefile 
 *   COMPILE_OPTS += -DTTC_LED1=PIN_PA7
 *
 * Source
 *   ttc_Port_t Port;
 *   ttc_loadPort(&Port, TTC_LED1);
 *   ttc_initPort(Port.GPIOx, Port.Pin, tgm_output_push_pull);
 */
void ttc_loadPort(ttc_Port_t* Port, ttc_GPIO_BANK GPIOx, u32_t Pin);

/* read current value of input pin
 * @param Port  filled out struct as being returned by ttc_loadPort()
 */
bool ttc_portGet(ttc_Port_t* Port);

/* set output pin to logical one
 * @param Port  filled out struct as being returned by ttc_loadPort()
 */
void ttc_portSet(ttc_Port_t* Port);

/* set output pin to logical zero
 * @param Port  filled out struct as being returned by ttc_loadPort()
 */
void ttc_portClr(ttc_Port_t* Port);

//} Prototypes

#endif
