#ifndef TTC_GPIO_TYPES_H
#define TTC_GPIO_TYPES_H

/*{ ttc_gpio_types.h ***********************************************
 *
 * Written by Gregor Rebel 2010-2012
 *
 * Basic defines, enums and structures.
 *
 * Include this file from 
 * architecture independent header files (ttc_XXX.h) and 
 * architecture depend      header files (e.g. stm32_XXX.h)
 * 
}*/
//{ includes

// #include "ttc_basic.h"

//}includes
//{ Structures/ Enums ****************************************************

typedef enum {   // ttc_gpio_mode_e
  tgm_None,                           // 0
  tgm_analog_in,                      // Analog input to ADC
  tgm_input_floating,                 // Digital input, High-Z
  tgm_input_pull_down,                // Digital input, resistor to GND active
  tgm_input_pull_up,                  // Digital input, resistor to VCC active
  tgm_output_open_drain,              // Digital output, can only pull to GND (to low)
  tgm_output_push_pull,               // Digital output, can pull to GND and VCC (to high and low)
  tgm_alternate_function_push_pull,   // Digital output from functional unit other than GPIO
  tgm_alternate_function_open_drain,  // Digital output from functional unit other than GPIO
  tgm_Unknown                         // Error condition
} ttc_gpio_mode_e;
typedef enum {   // ttc_gpio_speed_e
  tgs_None,
  tgs_Min,           // supported by: stm32
  tgs_2MHz,          // supported by: stm32
  tgs_10MHz,         // supported by: stm32
  tgs_50MHz,         // supported by: stm32
  tgs_Max,           // supported by: stm32
  tgs_Unknown
} ttc_gpio_speed_e;

//}Structures/ Enums
//{ includes2

#ifdef TARGET_ARCHITECTURE_STM32
#include "stm32_gpio.h"
#endif

// #include "ttc_basic.h"

//}includes2
//{ Defines


//} Defines
//{ Check if all required symbols are for current architecture ***********


/*{ Universal GPIO pin assignment
 * 
 * The shortcuts defined here provide an architecture independent way of
 * defining GPIO bank + pin in one single statement.
 * Each defined statement expands to two comma separated values: ttc_GPIO_BANK, PinNo
 * ttc_GPIO_BANK is then resolved according to current activate cpu type.
 *
 * Example (stm32):
 * PA3 -> GPIOA,3
 * 
 */
 
#ifndef PIN_PA0 //{
  #error Missing GPIO pin definitions for current Architecture!
/* Example (stm32)

//{ Define shortcuts for all pins of GPIO Bank A
#define PIN_PA0  GPIOA,0
#define PIN_PA1  GPIOA,1
#define PIN_PA2  GPIOA,2
#define PIN_PA3  GPIOA,3
#define PIN_PA4  GPIOA,4
#define PIN_PA5  GPIOA,5
#define PIN_PA6  GPIOA,6
#define PIN_PA7  GPIOA,7
#define PIN_PA8  GPIOA,8
#define PIN_PA9  GPIOA,9
#define PIN_PA10 GPIOA,10
#define PIN_PA11 GPIOA,11
#define PIN_PA12 GPIOA,12
#define PIN_PA13 GPIOA,13
#define PIN_PA14 GPIOA,14
#define PIN_PA15 GPIOA,15
//}BankA
//{ Define shortcuts for all pins of GPIO Bank B
#define PIN_PB0  GPIOB,0
#define PIN_PB1  GPIOB,1
#define PIN_PB2  GPIOB,2
#define PIN_PB3  GPIOB,3
#define PIN_PB4  GPIOB,4
#define PIN_PB5  GPIOB,5
#define PIN_PB6  GPIOB,6
#define PIN_PB7  GPIOB,7
#define PIN_PB8  GPIOB,8
#define PIN_PB9  GPIOB,9
#define PIN_PB10 GPIOB,10
#define PIN_PB11 GPIOB,11
#define PIN_PB12 GPIOB,12
#define PIN_PB13 GPIOB,13
#define PIN_PB14 GPIOB,14
#define PIN_PB15 GPIOB,15
//}BankB
...

*/
#endif //}
#ifndef ttc_GPIO_BANK //{

  #error Missing definition of ttc_GPIO_BANK for current Architecture!
/* Example (stm32)
 
#define ttc_GPIO_BANK  GPIO_TypeDef*

*/
#endif //}

//}Universal GPIO

#ifndef ttc_Port_t //{
  #error Missing definition of ttc_Port_t for current architecture!
  
/* Example definition of ttc_Port_t

typedef struct {
    GPIO_TypeDef* GPIOx;
    u8_t Pin;
} __attribute__((__packed__)) stm32_Port_t;

#define ttc_Port_t stm32_Port_t

*/

#endif //}

//}
#endif
