/*{ ttc_task.c ***********************************************
 
 * Written by Gregor Rebel 2010-2012
 *
 * Scheduler and architecture independent multitasking support.
 * 
}*/
#include "ttc_task.h"

void programCalibrateUSleep() {
  
#ifdef TTC_LED1
  ttc_Port_t Port_LED1;
  ttc_initPort(&Port_LED1, TTC_LED1, tgm_output_push_pull);
  
  while (1) {
    ttc_portSet(&Port_LED1);
    uSleep(1000);           // 1ms high time
    ttc_portClr(&Port_LED1);

    uSleep(100);            // 100us low time

  }
#else
  Assert(0, ec_UNKNOWN); // programCalibrateUSleep() requires definition of TTC_LED1! (E.g.; #define TTC_LED1 PA7)
#endif

}
void mSleep(u32_t Delay) {

    for (;Delay > 0; Delay--)
      uSleep(1000);
}
void uSleep(u32_t Delay) {

    Delay *= configTICK_RATE_HZ; // smallest delay depends on tick rate <-- test setting via programCalibrateUSleep()
    Delay /= 1000000;

    while (Delay > 0xffff) { // split long delays into smaller ones
      #ifdef EXTENSION_300_scheduler_free_rtos
        vTaskDelay(Delay & 0xffff);
      #endif
      Delay -= 0xffff;
    }
    #ifdef EXTENSION_300_scheduler_free_rtos
      vTaskDelay(Delay);
    #endif
}
void ttc_task_msleep(u32_t Delay) { mSleep(Delay); }
void ttc_task_usleep(u32_t Delay) { uSleep(Delay); }
void ttc_task_create( void TaskFunction(void*), const signed char* TaskName, u16_t StackSize, void* Argument, u8_t Priority, void* Handle) {

  #ifdef EXTENSION_300_scheduler_free_rtos
    #define IMPLEMENTATION_ttc_task_create=FreeRTOS
    xTaskCreate( (pdTASK_CODE) TaskFunction,
                 (const signed char *) TaskName,
                 (unsigned short ) StackSize,
                 Argument,
                 (unsigned portBASE_TYPE) Priority,
                 (xTaskHandle*) Handle
               );
  #endif

  // no scheduler available: use plain function call
  #ifndef IMPLEMENTATION_ttc_task_create
    #warning ttc_task_create() will use plain function calls
    TaskFunction->(Argument);
  #endif
}
void ttc_task_yield() {
#ifdef EXTENSION_300_scheduler_free_rtos
    taskYIELD();
#endif
}
void ttc_task_begin_criticalsection() {
    portENTER_CRITICAL();
}
void ttc_task_end_criticalsection() {
    portEXIT_CRITICAL();
}
