/*{ ttc_usart::.c ************************************************
 
                      The ToolChain
                      
   Device independent support for 
   Universal Synchronous Asynchronous Receiver and Transmitters (USARTs)
   
   written by Gregor Rebel 2012
   
 
}*/

#include "ttc_usart.h"

//{ Function definitions *************************************************

inline                  u8_t ttc_usart_get_max_index() {

    return TTC_AMOUNT_USARTS; // -> ttc_usart_types.h
}
inline ttc_usart_errorcode_e ttc_usart_get_defaults(u8_t USART_Index, ttc_usart_generic_t* USART_Generic) {
    memset(USART_Generic, 0, sizeof(USART_Generic));
    
#ifdef TARGET_ARCHITECTURE_STM32
    return stm32_usart_get_defaults(USART_Index, USART_Generic);
#endif

  return tue_NotImplemented;
}
inline ttc_usart_errorcode_e ttc_usart_get_features(u8_t USART_Index, ttc_usart_generic_t* USART_Generic) {
    memset(USART_Generic, 0, sizeof(USART_Generic));
    
#ifdef TARGET_ARCHITECTURE_STM32
    return stm32_usart_get_features(USART_Index, USART_Generic);
#endif

  return tue_NotImplemented;
}
inline ttc_usart_errorcode_e ttc_usart_init(u8_t USART_Index, ttc_usart_generic_t* USART_Generic, ttc_usart_architecture_t* USART_Arch) {
    memset(USART_Arch, 0, sizeof(USART_Arch));

#ifdef TARGET_ARCHITECTURE_STM32
  return stm32_usart_init(USART_Index, USART_Generic, (stm32_usart_architecture_t*) USART_Arch);
#endif

  return tue_NotImplemented;
}
inline ttc_usart_errorcode_e ttc_usart_send_raw(u8_t USART_Index, const char* Buffer, u16_t Amount) {
  
#ifdef TARGET_ARCHITECTURE_STM32
  return stm32_usart_send_raw(USART_Index, Buffer, Amount);
#endif

  return tue_NotImplemented;
}
inline ttc_usart_errorcode_e ttc_usart_send_string(u8_t USART_Index, const char* Buffer, u16_t MaxLength) {
  
#ifdef TARGET_ARCHITECTURE_STM32
  return stm32_usart_send_string(USART_Index, Buffer, MaxLength);
#endif

  return tue_NotImplemented;
}
inline ttc_usart_errorcode_e ttc_usart_send_word(u8_t USART_Index, const u16_t Word) {

#ifdef TARGET_ARCHITECTURE_STM32
  return stm32_usart_send_word(USART_Index, Word);
#endif

  return tue_NotImplemented;
}
inline ttc_usart_errorcode_e ttc_usart_read_word(u8_t USART_Index, u16_t* Word, u32_t TimeOut) {

#ifdef TARGET_ARCHITECTURE_STM32
  return stm32_usart_read_word(USART_Index, Word, TimeOut);
#endif

  return tue_NotImplemented;
}
inline ttc_usart_errorcode_e ttc_usart_read_byte(u8_t USART_Index, char* Byte, u32_t TimeOut) {

#ifdef TARGET_ARCHITECTURE_STM32
  return stm32_usart_read_byte(USART_Index, Byte, TimeOut);
#endif

  return tue_NotImplemented;
}

//} Function definitions
