#ifndef TTC_USART_TYPES_H
#define TTC_USART_TYPES_H

/*{ ttc_usart_types.h ***********************************************
 *
 * Written by Gregor Rebel 2010-2012
 *
 * Basic defines, enums and structures.
 *
 * Include this file from 
 * architecture independent header files (ttc_XXX.h) and 
 * architecture depend      header files (e.g. stm32_XXX.h)
 * 
}*/
#include "ttc_basic.h"

//{ Defines/ TypeDefs ****************************************************

// TTC_USARTn has to be defined as constant by makefile.100_board_*
#ifdef TTC_USART5
  #ifndef TTC_USART4
    #error TTC_USART5 is defined, but not TTC_USART4 - all lower TTC_USARTn must be defined!
  #endif
  #ifndef TTC_USART3
    #error TTC_USART5 is defined, but not TTC_USART3 - all lower TTC_USARTn must be defined!
  #endif
  #ifndef TTC_USART2
    #error TTC_USART5 is defined, but not TTC_USART2 - all lower TTC_USARTn must be defined!
  #endif
  #ifndef TTC_USART1
    #error TTC_USART5 is defined, but not TTC_USART1 - all lower TTC_USARTn must be defined!
  #endif

  #define TTC_AMOUNT_USARTS 5
#else
  #ifdef TTC_USART4
    #define TTC_AMOUNT_USARTS 4

    #ifndef TTC_USART3
      #error TTC_USART5 is defined, but not TTC_USART3 - all lower TTC_USARTn must be defined!
    #endif
    #ifndef TTC_USART2
      #error TTC_USART5 is defined, but not TTC_USART2 - all lower TTC_USARTn must be defined!
    #endif
    #ifndef TTC_USART1
      #error TTC_USART5 is defined, but not TTC_USART1 - all lower TTC_USARTn must be defined!
    #endif
  #else
    #ifdef TTC_USART3
    
      #ifndef TTC_USART2
        #error TTC_USART5 is defined, but not TTC_USART2 - all lower TTC_USARTn must be defined!
      #endif
      #ifndef TTC_USART1
        #error TTC_USART5 is defined, but not TTC_USART1 - all lower TTC_USARTn must be defined!
      #endif
    
      #define TTC_AMOUNT_USARTS 3
    #else
      #ifdef TTC_USART2

        #ifndef TTC_USART1
          #error TTC_USART5 is defined, but not TTC_USART1 - all lower TTC_USARTn must be defined!
        #endif

        #define TTC_AMOUNT_USARTS 2
      #else
        #ifdef TTC_USART1
          #define TTC_AMOUNT_USARTS 1
        #else
          #define TTC_AMOUNT_USARTS 0
        #endif
      #endif
    #endif
  #endif
#endif

//} Defines
//{ Structures/ Enums 1 **************************************************

typedef enum {   // ttc_usart_errorcode_e
  tue_OK,                // =0: no error
  tue_TimeOut,           // timeout occured in called function
  tue_DeviceNotFound,    // adressed USART device not available in current uC
  tue_NotImplemented,    // function has no implementation for current architecture
  tue_InvalidArgument,   // general argument error
  tue_InvalidBaudRate,   // given baud rate not suitable for adressed USART
  tue_InvalidWordSize,   // given word size not suitable for adressed USART
  tue_InvalidStopBits,   // given amount of stop bits not suitable for adressed USART
  tue_No_ControlParity,  // feature not available: check incoming parity bit
  tue_No_Receive,        // feature not available: activate receiver 
  tue_No_RtsCts,         // feature not available: Hardware Flow Control
  tue_No_SingleWire,     // feature not available: single-wire half-duplex
  tue_No_SmartCard,      // feature not available: SmartCard mode
  tue_No_Synchronous,    // feature not available: Synchronous Communication
  tue_No_Transmit,       // feature not available: activate transmitter
  tue_No_TransmitParity, // feature not available: transmit parity bit
  tue_InvalidPinConfig,  // one or more pins have been configured incorrectly -> check makefile for TTC_USARTxxx defines
  tue_DeviceNotReady,    // choosen device has not been initialized properly
  
  tue_UnknownError
} ttc_usart_errorcode_e;

typedef struct { // architecture independent configuration data of single USART
  union  {
    u32_t All;
    struct {
      // Bits
      unsigned ControlParity    : 1; // =1: check incoming parity bit
      unsigned IrDA             : 1; // =1: activate infrared mode 
      unsigned IrDALowPower     : 1; // =1: activate infrared low power mode 
      unsigned IrqOnCts         : 1; // =1: interrupt is generated when CTS=1  
      unsigned IrqOnError       : 1; // =1: interrupt is generated in case of any error  
      unsigned IrqOnIdle        : 1; // =1: interrupt is generated when USART is idle  
      unsigned IrqOnTxComplete  : 1; // =1: interrupt is generated when transmission has completed
      unsigned IrqOnParityError : 1; // =1: interrupt is generated when parity error occurs
      unsigned LIN              : 1; // =1: activate Local Interconnection Network mode (serial bus)
      unsigned ParityEven       : 1; // =1: activate parity checking with even parity
      unsigned ParityOdd        : 1; // =1: activate parity checking with odd parity
      unsigned Receive          : 1; // =1: activate receiver via RxD
      unsigned Cts              : 1; // =1: data is only sent when Cts goes low  
      unsigned Rts              : 1; // =1: data is only requested when receive buffer is empty  
      unsigned RtsCts           : 1; // =1: activate hardware flow-control via RTS/CTS wires
      unsigned RxDMA            : 1; // =1: enable DMA for Receiver 
      unsigned SendBreaks       : 1; // =1: activate software flow-control via break characters 
      unsigned SingleWire       : 1; // =1: use single-wire half-duplex
      unsigned SmartCard        : 1; // =1: activate smartcard mode 
      unsigned SmartCardNACK    : 1; // =1: transmission during parity error enabled 
      unsigned Synchronous      : 1; // =1: activate synchronous mode (requires additional clock wire)  
      unsigned Transmit         : 1; // =1: activate transmitter via TxD
      unsigned TransmitParity   : 1; // =1: transmit parity bit
      unsigned TxDMA            : 1; // =1: enable DMA for Transmitter  
      
      unsigned Reserved1        : 8; // pad to 32 bits
    };
  } Flags;
  
  unsigned WordLength         : 4; // size of each character (7..9 Bits)
  unsigned HalfStopBits       : 3; // amount of half stop bits (1..4 * 0.5 Bits -> 0.5 .. 2 StopBits)
  unsigned Reserved2          : 1; // pad to 1 Byte
  u8_t     Layout;                 // select pin layout to use (some uC allow pin remapping)
  
  u32_t    BaudRate;               // symbol rate of connection
  
} __attribute__((__packed__)) ttc_usart_generic_t;

//}

#endif // TTC_USART_TYPES_H
