/*{ Template_FreeRTOS::.c ************************************************

     Several different boxes run over the screen.
     Each box is controlled by one task.
     
     written 2012 by Ioanna Tsekoura, Gregor Rebel
     
}*/

#include "example_lcd_boxes.h"

u16_t LCD_Width  = 240;
u16_t LCD_Height = 320;
u16_t Color_Background = LCD_COLOR_YELLOW;

//{ Function definitions *************************************************

void lcd_boxes_initPorts() {

    //{ init_LCD Start
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART2, ENABLE);

    GPIO_InitTypeDef GPIO_InitStructure;
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_All;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;
    GPIO_Init(GPIOA, &GPIO_InitStructure);
    GPIO_Init(GPIOB, &GPIO_InitStructure);
    GPIO_Init(GPIOC, &GPIO_InitStructure);
    GPIO_Init(GPIOD, &GPIO_InitStructure);
    GPIO_Init(GPIOE, &GPIO_InitStructure);
    GPIO_Init(GPIOF, &GPIO_InitStructure);
    GPIO_Init(GPIOG, &GPIO_InitStructure);

    /* Enable the FSMC Clock */
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_FSMC, ENABLE);

    LCD_Init();

    //} init_LCD Ende

    LCD_SetBacklight(1);
    LCD_Clear(Color_Background);

}
void lcd_start_boxes() {

    xQueueHandle DrawQueue = xQueueCreate(10, sizeof(DrawData_t));
    Assert(NULL != DrawQueue, ec_Malloc);

    lcd_boxes_initPorts();
    ttc_srand(32);

    for (int I = 1; I < 30; I++) {
        DrawData_t* Box = newRandomBox(DrawQueue);
    }

    portBASE_TYPE Task2 = ttc_task_create(tDrawIt,                                 // function to start as thread
                                      (const signed char*) "DrawsRect",        // thread name (just for debugging)
                                      (size_t) 1 * configMINIMAL_STACK_SIZE,   // stack size
                                      (void *) DrawQueue,                      // passed as argument to taskLEDs()
                                      (unsigned portBASE_TYPE) 1,              // task priority (higher values mean more process time)
                                      (xTaskHandle*) NULL                      // can return a handle to created task
                                      );
    Assert(0 != Task2, ec_Malloc);
}
DrawData_t* newRandomBox(xQueueHandle DrawingQueue) {
    return newBox( DrawingQueue,
                   LCD_RGB_TO_LCDCOLOR(ttc_rand() & 0xff, ttc_rand() & 0xff, ttc_rand() & 0xff),
                   1 + ttc_rand() % LCD_Width,
                   1 + ttc_rand() % LCD_Height,
                   1 + ttc_rand() % 20,
                   1 + ttc_rand() % 20,
                   1 + ttc_rand() % 20,
                   1 + ttc_rand() % 20
                 );
}
DrawData_t* newBox(xQueueHandle DrawingQueue, u16_t Color, s16_t X, s16_t Y, u8_t Width, u8_t Height, s8_t SpeedX, s8_t SpeedY) {
    drawTaskArgs_t* NewBox = (drawTaskArgs_t*) pvPortMalloc( sizeof(drawTaskArgs_t) );

    if (NewBox != NULL) {
        NewBox->DrawQueue = DrawingQueue;
        NewBox->Box2Draw.Type = ddt_MovingBox;
        NewBox->Box2Draw.Polygon.MovingBox.Color  = Color;
        NewBox->Box2Draw.Polygon.MovingBox.PosX   = X;
        NewBox->Box2Draw.Polygon.MovingBox.PosY   = Y;
        NewBox->Box2Draw.Polygon.MovingBox.Width  = Width;
        NewBox->Box2Draw.Polygon.MovingBox.Height = Height;
        NewBox->Box2Draw.Polygon.MovingBox.SpeedX = SpeedX;
        NewBox->Box2Draw.Polygon.MovingBox.SpeedY = SpeedY;

        portBASE_TYPE Task = ttc_task_create(tMoveBox,                             // function to start as thread
                                         (const signed char*) NULL,             // thread name (just for debugging)
                                         (size_t) 2 * configMINIMAL_STACK_SIZE, // stack size
                                         (void *) NewBox,                       // passed as argument to task-function
                                         (unsigned portBASE_TYPE) 1,            // task priority (higher values mean more process time)
                                         (xTaskHandle*) NULL                    // can return a handle to created task
                                         );
        Assert(Task != 0, ec_Malloc); 
    }
    return &(NewBox->Box2Draw);
}
void tMoveBox(void *pvParameters) {

    drawTaskArgs_t* Arguments = (drawTaskArgs_t*) pvParameters;
    xQueueHandle DrawQueue = Arguments->DrawQueue;

    Assert(NULL != DrawQueue, ec_InvalidArgument);

    DrawData_t* DrawMovingBox = &(Arguments->Box2Draw);
    elb_MovingBox_t* MovingBox = &(DrawMovingBox->Polygon.MovingBox);

    DrawData_t DrawClearBox;
    DrawClearBox.Type = ddt_MovingBox;
    elb_MovingBox_t* ClearBox = &(DrawClearBox.Polygon.MovingBox);
    ClearBox->Color  = Color_Background;
    ClearBox->SpeedX = 0;
    ClearBox->SpeedY = 0;

    DrawData_t DrawClearLPolygon;
    DrawClearLPolygon.Type = ddt_L_Polygon;
    elb_L_Polygon_t* ClearLPolygon = &(DrawClearLPolygon.Polygon.L_Polygon);
    ClearLPolygon->Color  = Color_Background;

    DrawData_t DrawLPolygon;
    DrawLPolygon.Type = ddt_L_Polygon;
    elb_L_Polygon_t* LPolygon = &(DrawLPolygon.Polygon.L_Polygon);

    queueInsert(DrawQueue, DrawMovingBox); // Drawing of the Box(es) in the initial position(s).
    u16_t CountDown = 0;
    u8_t UseImplementation = 0;

    while (1) {

        int PosX = MovingBox->PosX;
        int PosY = MovingBox->PosY;
        int NewPosX = PosX + MovingBox->SpeedX;
        int NewPosY = PosY + MovingBox->SpeedY;

        int Border_Right = LCD_Width  - MovingBox->Width;
        int Border_Lower = LCD_Height - MovingBox->Height;

        if (NewPosX > Border_Right) {
            MovingBox->SpeedX = -MovingBox->SpeedX;
            int Excess = NewPosX - Border_Right;
            NewPosX -= Excess;
        }
        else if (NewPosX < 0) {
            MovingBox->SpeedX = -MovingBox->SpeedX;
            NewPosX = -NewPosX;
        }
        if (NewPosY > Border_Lower) {
            MovingBox->SpeedY = -MovingBox->SpeedY;
            int Excess = NewPosY - Border_Lower;
            NewPosY -= Excess;
        }
        else if (NewPosY < 0) {
            MovingBox->SpeedY = -MovingBox->SpeedY;
            NewPosY = -NewPosY;
        }

        if (CountDown == 0) {
            UseImplementation = 1 - UseImplementation; // switch implementation
            CountDown = 100; // switch every 3 seconds
            ClearBox->PosX = 0;
            ClearBox->PosY = 0;
            ClearBox->Width  = 5;
            ClearBox->Height = 5;
            if (UseImplementation)  ClearBox->Color  = LCD_COLOR_MAGENTA;
            else                    ClearBox->Color  = LCD_COLOR_GREEN;
            queueInsert(DrawQueue, &DrawClearBox);
            ClearBox->Color  = Color_Background;
        }
        CountDown--;
        if (UseImplementation) { // 1st implementation, clear the previous box and redraw all of it
            ClearBox->PosX = PosX;
            ClearBox->PosY = PosY;
            ClearBox->Width  = MovingBox->Width;
            ClearBox->Height = MovingBox->Height;
            queueInsert(DrawQueue, &DrawClearBox);

            MovingBox->PosX = NewPosX;
            MovingBox->PosY = NewPosY;
            queueInsert(DrawQueue, DrawMovingBox); // draw box as indicator of implementation switch
        }
        else {                   // 2nd implementation: clear as few pixels as possible

            if ((MovingBox->PosX==NewPosX)&&(MovingBox->PosY==NewPosY)) {
                // clear nothing
                MovingBox->PosX=NewPosX;
                MovingBox->PosY=NewPosY;
                queueInsert(DrawQueue, DrawMovingBox);
            }
            else if ((MovingBox->PosX==NewPosX)&&(MovingBox->PosY<NewPosY)) {
                if ((MovingBox->PosY+MovingBox->Height)<NewPosY) {
                    ClearBox->PosX   = PosX;
                    ClearBox->PosY   = PosY;
                    ClearBox->Width  = MovingBox->Width;
                    ClearBox->Height = MovingBox->Height;
                    queueInsert(DrawQueue, &DrawClearBox);

                    MovingBox->PosX=NewPosX;
                    MovingBox->PosY=NewPosY;
                    queueInsert(DrawQueue, DrawMovingBox);
                }
                else {
                    ClearBox->PosX   = PosX;
                    ClearBox->PosY   = PosY;
                    ClearBox->Width  = MovingBox->Width;
                    ClearBox->Height = NewPosY - PosY;
                    queueInsert(DrawQueue, &DrawClearBox);

                    MovingBox->PosX=NewPosX;
                    MovingBox->PosY=NewPosY;
                    queueInsert(DrawQueue, DrawMovingBox);
                }
            }
            else if ((MovingBox->PosX==NewPosX)&&(MovingBox->PosY>NewPosY)) {
                if ((NewPosY+MovingBox->Height)<MovingBox->PosY) {
                    ClearBox->PosX   = PosX;
                    ClearBox->PosY   = PosY;
                    ClearBox->Width  = MovingBox->Width;
                    ClearBox->Height = MovingBox->Height;
                    queueInsert(DrawQueue, &DrawClearBox);

                    MovingBox->PosX=NewPosX;
                    MovingBox->PosY=NewPosY;
                    queueInsert(DrawQueue, DrawMovingBox);
                }
                else {
                    ClearBox->PosX   = PosX;
                    ClearBox->PosY   = NewPosY+MovingBox->Height;
                    ClearBox->Width  = MovingBox->Width;
                    ClearBox->Height = PosY-NewPosY;
                    queueInsert(DrawQueue, &DrawClearBox);

                    MovingBox->PosX=NewPosX;
                    MovingBox->PosY=NewPosY;
                    queueInsert(DrawQueue, DrawMovingBox);
                }
            }
            else if ((MovingBox->PosX<NewPosX)&&(MovingBox->PosY==NewPosY)) {
                if ((MovingBox->PosX+MovingBox->Width)<NewPosX) {
                    ClearBox->PosX   = PosX;
                    ClearBox->PosY   = PosY;
                    ClearBox->Width  = MovingBox->Width;
                    ClearBox->Height = MovingBox->Height;
                    queueInsert(DrawQueue, &DrawClearBox);

                    MovingBox->PosX=NewPosX;
                    MovingBox->PosY=NewPosY;
                    queueInsert(DrawQueue, DrawMovingBox);
                }
                else {
                    ClearBox->PosX   = PosX;
                    ClearBox->PosY   = PosY;
                    ClearBox->Width  = NewPosX-PosX;
                    ClearBox->Height = MovingBox->Height;
                    queueInsert(DrawQueue, &DrawClearBox);

                    MovingBox->PosX=NewPosX;
                    MovingBox->PosY=NewPosY;
                    queueInsert(DrawQueue, DrawMovingBox);
                }
            }
            else if ((MovingBox->PosX>NewPosX)&&(MovingBox->PosY==NewPosY)) {
                if ((NewPosX+MovingBox->Width)<MovingBox->PosX) {
                    ClearBox->PosX   = PosX;
                    ClearBox->PosY   = PosY;
                    ClearBox->Width  = MovingBox->Width;
                    ClearBox->Height = MovingBox->Height;
                    queueInsert(DrawQueue, &DrawClearBox);

                    MovingBox->PosX=NewPosX;
                    MovingBox->PosY=NewPosY;
                    queueInsert(DrawQueue, DrawMovingBox);
                }
                else {
                    ClearBox->PosX   = NewPosX+MovingBox->Width;
                    ClearBox->PosY   = PosY;
                    ClearBox->Width  = PosX-NewPosX;
                    ClearBox->Height = MovingBox->Height;
                    queueInsert(DrawQueue, &DrawClearBox);

                    MovingBox->PosX=NewPosX;
                    MovingBox->PosY=NewPosY;
                    queueInsert(DrawQueue, DrawMovingBox);
                }
            }

            else if ((MovingBox->PosX<NewPosX)&&(MovingBox->PosY<NewPosY)) { /*Left to Right. Top to Bottom.*/
                if ((NewPosX>MovingBox->PosX+MovingBox->Width)||(NewPosY>MovingBox->PosY+MovingBox->Height)) {
                    ClearBox->PosX   = PosX;
                    ClearBox->PosY   = PosY;
                    ClearBox->Width  = MovingBox->Width;
                    ClearBox->Height = MovingBox->Height;
                    queueInsert(DrawQueue, &DrawClearBox);

                    MovingBox->PosX=NewPosX;
                    MovingBox->PosY=NewPosY;
                    queueInsert(DrawQueue, DrawMovingBox);
                }
                else {
                    ClearLPolygon->ExternalX = PosX;
                    ClearLPolygon->ExternalY = PosY;
                    ClearLPolygon->Width     = MovingBox->Width;
                    ClearLPolygon->Height    = MovingBox->Height;
                    ClearLPolygon->InternalX = NewPosX;
                    ClearLPolygon->InternalY = NewPosY;
                    queueInsert(DrawQueue, &DrawClearLPolygon);

                    LPolygon->Color = MovingBox->Color;
                    LPolygon->ExternalX = NewPosX+MovingBox->Width;
                    LPolygon->ExternalY = NewPosY+MovingBox->Height;
                    LPolygon->Width     = MovingBox->Width;
                    LPolygon->Height    = MovingBox->Height;
                    LPolygon->InternalX = PosX+MovingBox->Width;
                    LPolygon->InternalY = PosY+MovingBox->Height;
                    queueInsert(DrawQueue, &DrawLPolygon);
                    MovingBox->PosX=NewPosX;
                    MovingBox->PosY=NewPosY;
                }
            }
            else if ((MovingBox->PosX<NewPosX)&&(MovingBox->PosY>NewPosY)) { /*Left to Right. Bottom to Top*/
                if ((NewPosX>MovingBox->PosX+MovingBox->Width)||((NewPosY+MovingBox->Height)<MovingBox->PosY)) {

                    ClearBox->PosX   = PosX;
                    ClearBox->PosY   = PosY;
                    ClearBox->Width  = MovingBox->Width;
                    ClearBox->Height = MovingBox->Height;
                    queueInsert(DrawQueue, &DrawClearBox);

                    MovingBox->PosX=NewPosX;
                    MovingBox->PosY=NewPosY;
                    queueInsert(DrawQueue, DrawMovingBox);
                }
                else {
                    ClearLPolygon->ExternalX = PosX;
                    ClearLPolygon->ExternalY = PosY+MovingBox->Height;
                    ClearLPolygon->Width     = MovingBox->Width;
                    ClearLPolygon->Height    = MovingBox->Height;
                    ClearLPolygon->InternalX = NewPosX;
                    ClearLPolygon->InternalY = NewPosY+MovingBox->Height;
                    queueInsert(DrawQueue, &DrawClearLPolygon);

                    LPolygon->Color = MovingBox->Color;
                    LPolygon->ExternalX = NewPosX+MovingBox->Width;
                    LPolygon->ExternalY = NewPosY;
                    LPolygon->Width     = MovingBox->Width;
                    LPolygon->Height    = MovingBox->Height;
                    LPolygon->InternalX = PosX+MovingBox->Width;
                    LPolygon->InternalY = PosY;
                    queueInsert(DrawQueue, &DrawLPolygon);
                    MovingBox->PosX=NewPosX;
                    MovingBox->PosY=NewPosY;
                }
            }
            else if ((MovingBox->PosX>NewPosX)&&(MovingBox->PosY<NewPosY)) { /*Right to Left. Top to Bottom*/
                if (((NewPosX+MovingBox->Width)<MovingBox->PosX)||(NewPosY>MovingBox->PosY+MovingBox->Height)) {
                    ClearBox->PosX   = PosX;
                    ClearBox->PosY   = PosY;
                    ClearBox->Width  = MovingBox->Width;
                    ClearBox->Height = MovingBox->Height;
                    queueInsert(DrawQueue, &DrawClearBox);

                    MovingBox->PosX=NewPosX;
                    MovingBox->PosY=NewPosY;
                    queueInsert(DrawQueue, DrawMovingBox);
                }
                else {
                    ClearLPolygon->ExternalX = PosX+MovingBox->Width;
                    ClearLPolygon->ExternalY = PosY;
                    ClearLPolygon->Width     = MovingBox->Width;
                    ClearLPolygon->Height    = MovingBox->Height;
                    ClearLPolygon->InternalX = NewPosX+MovingBox->Width;
                    ClearLPolygon->InternalY = NewPosY;
                    queueInsert(DrawQueue, &DrawClearLPolygon);

                    LPolygon->Color     = MovingBox->Color;
                    LPolygon->ExternalX = NewPosX;
                    LPolygon->ExternalY = NewPosY+MovingBox->Height;
                    LPolygon->Width     = MovingBox->Width;
                    LPolygon->Height    = MovingBox->Height;
                    LPolygon->InternalX = PosX;
                    LPolygon->InternalY = PosY+MovingBox->Height;
                    queueInsert(DrawQueue, &DrawLPolygon);
                    MovingBox->PosX=NewPosX;
                    MovingBox->PosY=NewPosY;
                }
            }
            else if ((MovingBox->PosX>NewPosX)&&(MovingBox->PosY>NewPosY)) {
                if (((NewPosX+MovingBox->Width)<MovingBox->PosX)||((NewPosY+MovingBox->Height)<MovingBox->PosY)) {
                    ClearBox->PosX   = PosX;
                    ClearBox->PosY   = PosY;
                    ClearBox->Width  = MovingBox->Width;
                    ClearBox->Height = MovingBox->Height;
                    queueInsert(DrawQueue, &DrawClearBox);

                    MovingBox->PosX=NewPosX;
                    MovingBox->PosY=NewPosY;
                    queueInsert(DrawQueue, DrawMovingBox);
                }
                else {
                    ClearLPolygon->ExternalX = PosX+MovingBox->Width;
                    ClearLPolygon->ExternalY = PosY+MovingBox->Height;
                    ClearLPolygon->Width     = MovingBox->Width;
                    ClearLPolygon->Height    = MovingBox->Height;
                    ClearLPolygon->InternalX = NewPosX+MovingBox->Width;
                    ClearLPolygon->InternalY = NewPosY+MovingBox->Height;
                    queueInsert(DrawQueue, &DrawClearLPolygon);

                    LPolygon->Color     = MovingBox->Color;
                    LPolygon->ExternalX = NewPosX;
                    LPolygon->ExternalY = NewPosY;
                    LPolygon->Width     = MovingBox->Width;
                    LPolygon->Height    = MovingBox->Height;
                    LPolygon->InternalX = PosX;
                    LPolygon->InternalY = PosY;
                    queueInsert(DrawQueue, &DrawLPolygon);
                    MovingBox->PosX=NewPosX;
                    MovingBox->PosY=NewPosY;
                }
            }
        }

        mSleep(30);
    }
}
void queueInsert(xQueueHandle DrawQueue, DrawData_t *Box) {

    while (xQueueSend(DrawQueue, Box, (portTickType) -1) != pdTRUE) 
      ttc_task_yield();

}
void tDrawIt(void *pvParameters) {
    xQueueHandle DrawQueue= (xQueueHandle) pvParameters;
    Assert(NULL != DrawQueue, ec_InvalidArgument);

    while (1) {
        DrawData_t Box2Draw;
        while ( xQueueReceive(DrawQueue, &Box2Draw, (portTickType) -1) ) {
            if (Box2Draw.Type == ddt_MovingBox) {
                LCD_DrawRect(Box2Draw.Polygon.MovingBox.PosX,
                             Box2Draw.Polygon.MovingBox.PosY,
                             Box2Draw.Polygon.MovingBox.Width,
                             Box2Draw.Polygon.MovingBox.Height,
                             Box2Draw.Polygon.MovingBox.Color
                             );
            }
            else {
                LCD_DrawLpolygon(Box2Draw.Polygon.L_Polygon.ExternalX,
                                 Box2Draw.Polygon.L_Polygon.ExternalY,
                                 Box2Draw.Polygon.L_Polygon.Width,
                                 Box2Draw.Polygon.L_Polygon.Height,
                                 Box2Draw.Polygon.L_Polygon.InternalX,
                                 Box2Draw.Polygon.L_Polygon.InternalY,
                                 Box2Draw.Polygon.L_Polygon.Color
                                );
            }
        }
        ttc_task_yield();
    }
}
void LCD_DrawLpolygon(int ExternalX, int ExternalY, int Width, int Height, int InternalX, int InternalY, u16 Color)
{
    int i;

    if ((ExternalX<InternalX)&&(ExternalY<InternalY)) {
        int k=ExternalY;
        for(i=0; i<Height; i++, k++) {
            u32 addr = k*0x100 + ExternalX;
            LCD_FillMem(addr, Color, (InternalX-ExternalX));
        }
        k=ExternalY;
        for(i=0; i<(InternalY-ExternalY); i++, k++) {
            u32 addr = k*0x100 + InternalX;
            LCD_FillMem(addr, Color, (ExternalX+Width-InternalX));
        }
    }
    else if ((ExternalX<InternalX)&&(ExternalY>InternalY)) {
        int k=ExternalY-Height;
        for(i=0; i<Height; i++, k++) {
            u32 addr = k*0x100 + ExternalX;
            LCD_FillMem(addr, Color, (InternalX-ExternalX));
        }
        k=InternalY;
        for(i=0; i<(ExternalY-InternalY); i++, k++) {
            u32 addr = k*0x100 + InternalX;
            LCD_FillMem(addr, Color, (ExternalX+Width-InternalX));
        }
    }
    else if ((ExternalX>InternalX)&&(ExternalY<InternalY)) {
        int k=ExternalY;
        for(i=0; i<(InternalY-ExternalY); i++, k++) {
            u32 addr = k*0x100 + ExternalX-Width;
            LCD_FillMem(addr, Color, Width);
        }
        k=InternalY;
        for(i=0; i<(ExternalY+Height-InternalY); i++, k++) {
            u32 addr = k*0x100 + InternalX;
            LCD_FillMem(addr, Color, (ExternalX-InternalX));
        }
    }
    else {
        int k=InternalY;
        for(i=0; i<(ExternalY-InternalY); i++, k++) {
            u32 addr = k*0x100 + ExternalX-Width;
            LCD_FillMem(addr, Color, Width);
        }
        k=ExternalY-Height;
        for(i=0; i<(InternalY+Height-ExternalY); i++, k++) {
            u32 addr = k*0x100 + InternalX;
            LCD_FillMem(addr, Color, (ExternalX-InternalX));
        }
    }
}

//} Function definitions
