/*{ ExampleThreading::.c ************************************************
 
 Example code demonstrating how to use inter thread communication.
 
}*/

#include "example_threading_semaphores.h"

#define AmountProducers 1
#define AmountConsumers 6

//{ Function definitions *************************************************

void threading_init() {
#ifdef EXTENSION_400_lcd_320x240_olimex

    if (1) { // init_LCD Start
        RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART2, ENABLE);

        GPIO_InitTypeDef GPIO_InitStructure;
        GPIO_InitStructure.GPIO_Pin = GPIO_Pin_All;
        GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;
        GPIO_Init(GPIOA, &GPIO_InitStructure);
        GPIO_Init(GPIOB, &GPIO_InitStructure);
        GPIO_Init(GPIOC, &GPIO_InitStructure);
        GPIO_Init(GPIOD, &GPIO_InitStructure);
        GPIO_Init(GPIOE, &GPIO_InitStructure);
        GPIO_Init(GPIOF, &GPIO_InitStructure);
        GPIO_Init(GPIOG, &GPIO_InitStructure);

#ifdef RCC_AHBPeriph_FSMC
        /* Enable the FSMC Clock */
        RCC_AHBPeriphClockCmd(RCC_AHBPeriph_FSMC, ENABLE);
#endif

        LCD_Init();
    }

    LCD_SetBacklight(1);
    u16_t BgColor = LCD_RGB_TO_LCDCOLOR(200,255,200);
    LCD_Clear(BgColor);

    //LCD_DrawChar(1,1, '>', LCD_COLOR_YELLOW, LCD_COLOR_BLACK);
    LCD_ConSetColor(BgColor, LCD_COLOR_BLACK);
#endif

}
void threading_start() {
    threading_init();

    static ets_ThreadInfo_t ThreadInfo; // must be static to survive end of this function!

    // Mutex being used to wait until task has copied its ThreadInfo
    ThreadInfo.Lock_Init = xSemaphoreCreateMutex();
    Assert(NULL != ThreadInfo.Lock_Init, ec_Malloc);

    // create counting semaphore Producer -> Consumer
    ThreadInfo.SemaphoreProducer2Consumer = xQueueCreateCountingSemaphore(30, 0);
    Assert(NULL != ThreadInfo.SemaphoreProducer2Consumer, ec_Malloc);

    // create queue Producer/Consumer -> Print
    ThreadInfo.QueuePrint = xQueueCreate(5, sizeof(et_PrintJob_t));
    Assert(NULL != ThreadInfo.QueuePrint, ec_Malloc);
    
    xSemaphoreTake(ThreadInfo.Lock_Init, 0);  // lock structure
    Assert(0 != ttc_task_create( taskPrint,                             //{ function to start as thread
                             (const signed char*) "Print",          // thread name (just for debugging)
                             (size_t) 4 * configMINIMAL_STACK_SIZE, // stack size
                             (void *) &ThreadInfo,                  // passed as argument to taskValue()
                             (unsigned portBASE_TYPE) 1,            // task priority (higher values mean more process time)
                             (xTaskHandle*) NULL                    // can return a handle to created task
                           ),
           ec_Malloc); //}
    while (! xSemaphoreTake(ThreadInfo.Lock_Init, (portTickType) -1) );  // wait until task has released structure

    ThreadInfo.DisplayRow = 5;

    for (int Amount = 0; Amount < AmountProducers; Amount++) {
        Assert(0 != ttc_task_create( taskProducer,                          //{ function to start as thread
                                 (const signed char*) "Producer",       // thread name (just for debugging)
                                 (size_t) 4 * configMINIMAL_STACK_SIZE, // stack size
                                 (void *) &ThreadInfo,                  // passed as argument to taskValue()
                                 (unsigned portBASE_TYPE) 1,            // task priority (higher values mean more process time)
                                 (xTaskHandle*) NULL                    // can return a handle to created task
                               ),
               ec_Malloc); //}

        while (! xSemaphoreTake(ThreadInfo.Lock_Init, (portTickType) -1) );  // wait until task has released structure
        ThreadInfo.DisplayRow++;
    }

    for (int Amount = 0; Amount < AmountConsumers; Amount++) {
        ttc_Port_t* Port = &(ThreadInfo.ConfigLED.Port);
        switch (Amount) {
#ifdef TTC_LED1
          case  0: ttc_initPort(Port, TTC_LED1, tgm_output_push_pull); break; // loads *Port with TTC_LED1
#endif
#ifdef TTC_LED2
          case  1: ttc_initPort(Port, TTC_LED2, tgm_output_push_pull); break; // loads *Port with TTC_LED2
#endif
          default: ttc_loadPort(Port, NULL, 0);                       break; // loads *Port with 0,0
        }

        Assert(0 != ttc_task_create( taskConsumer,                          //{ function to start as thread
                                 (const signed char*) "Consumer",       // thread name (just for debugging)
                                 (size_t) 4 * configMINIMAL_STACK_SIZE, // stack size
                                 (void *) &ThreadInfo,                  // passed as argument to taskValue()
                                 (unsigned portBASE_TYPE) 1,            // task priority (higher values mean more process time)
                                 (xTaskHandle*) NULL                    // can return a handle to created task
                                 ),
               ec_Malloc); //}
        while (! xSemaphoreTake(ThreadInfo.Lock_Init, (portTickType) -1) );  // wait until task has released structure
        ThreadInfo.DisplayRow++;
    }
    xSemaphoreGive(ThreadInfo.Lock_Init);

    queuePrintJob(ThreadInfo.QueuePrint, 4, 0, "Example");
    queuePrintJob(ThreadInfo.QueuePrint, 3, 1, "Semaphores");
    queuePrintJob(ThreadInfo.QueuePrint, 0, 3, "Q=");

}
void displayQueue(xSemaphoreHandle Semaphore, xQueueHandle QueuePrint) {
    char QueueText[11];
    char* Writer = QueueText;
    unsigned portBASE_TYPE QueueSize = uxQueueMessagesWaiting(Semaphore);
    u8_t I = 0;
    for (; (I < QueueSize/3) && (I < 10); I++)
        *Writer++ = '.';
    for (; I < 10; I++)
        *Writer++ = ' ';
    *Writer++ = 0;
    queuePrintJob(QueuePrint, 2, 3, QueueText);
}
void taskProducer(void *pvParameters) {
    ets_ThreadInfo_t* ThreadInfoPtr = (ets_ThreadInfo_t*) pvParameters;
    Assert(NULL != ThreadInfoPtr, ec_NULL);
    Assert(NULL != ThreadInfoPtr->SemaphoreProducer2Consumer, ec_InvalidArgument);
    Assert(NULL != ThreadInfoPtr->QueuePrint,                 ec_InvalidArgument);
    Assert(NULL != ThreadInfoPtr->Lock_Init,                  ec_InvalidArgument);

    ets_ThreadInfo_t ThreadInfo;
    memcpy(&ThreadInfo, pvParameters, sizeof(ets_ThreadInfo_t));
    xSemaphoreGive(ThreadInfo.Lock_Init); // task creator may now reuse *pvParameters

    queuePrintJob(ThreadInfo.QueuePrint, 2, ThreadInfo.DisplayRow, "P>");

    u8_t Value = 0; //(u16_t) rand() & 0xfff;
    while (1) {
        queuePrintJobV(ThreadInfo.QueuePrint, 6, ThreadInfo.DisplayRow, "+%i ", Value/10 + 1);
        for (int I = 0; I < Value; I+=10) {
            while (xSemaphoreGive(ThreadInfo.SemaphoreProducer2Consumer) != pdTRUE);
        }

        displayQueue(ThreadInfo.SemaphoreProducer2Consumer, ThreadInfo.QueuePrint);
        mSleep(1200);

        Value++;
        if (Value == 60) Value = 0;
    }
}
void taskConsumer(void *pvParameters) {
    Assert(NULL != pvParameters,                             ec_NULL);
    Assert(NULL != ((ets_ThreadInfo_t*) pvParameters)->SemaphoreProducer2Consumer, ec_InvalidArgument);
    Assert(NULL != ((ets_ThreadInfo_t*) pvParameters)->QueuePrint,                 ec_InvalidArgument);
    Assert(NULL != ((ets_ThreadInfo_t*) pvParameters)->Lock_Init,                  ec_InvalidArgument);

    ets_ThreadInfo_t ThreadInfo;
    memcpy(&ThreadInfo, pvParameters, sizeof(ets_ThreadInfo_t));
    xSemaphoreGive(ThreadInfo.Lock_Init); // task creator may now reuse *pvParameters

    queuePrintJob(ThreadInfo.QueuePrint, 2, ThreadInfo.DisplayRow, "C>");
    ttc_Port_t* Port_LED = &ThreadInfo.ConfigLED.Port;
    if (Port_LED->GPIOx == NULL) 
      Port_LED = NULL;              // port was not initialized: Dont use it

    u32_t Counter = 0;
    while (1) {
        if ( xSemaphoreTake(ThreadInfo.SemaphoreProducer2Consumer, (portTickType) -1) ) {
            Counter++;
            if (Port_LED) {
              for (int I = Counter; I >= 0; I--) {
                  ttc_portSet(Port_LED);
                  mSleep(400);
                  ttc_portClr(Port_LED);
                  mSleep(400);
              }
            }
#ifdef EXTENSION_400_lcd_320x240_olimex
            queuePrintJob(ThreadInfo.QueuePrint, 5, ThreadInfo.DisplayRow, "|");
            mSleep(400);
            queuePrintJob(ThreadInfo.QueuePrint, 5, ThreadInfo.DisplayRow, "-");
            mSleep(400);
            queuePrintJobV(ThreadInfo.QueuePrint, 7, ThreadInfo.DisplayRow, "%d", Counter);
            displayQueue(ThreadInfo.SemaphoreProducer2Consumer, ThreadInfo.QueuePrint);
#endif
        }
    }

    while (1) mSleep(1000);
}
void queuePrintJobV(xQueueHandle PrintQueue, s8_t Column, s8_t Row, const char* Text, s16_t Value) {
    et_PrintJob_t PrintJob;
    PrintJob.Column = Column;
    PrintJob.Row    = Row;
    PrintJob.Value  = Value;
    strncpy(PrintJob.Text, Text, 10);

    while (xQueueSend(PrintQueue, &PrintJob, (portTickType) -1) != pdTRUE);
}
void queuePrintJob(xQueueHandle PrintQueue, s8_t Column, s8_t Row, const char* Text) {
    queuePrintJobV(PrintQueue, Column, Row, Text, 0);
}
void taskPrint(void *pvParameters) {
    ets_ThreadInfo_t* ThreadInfoPtr = (ets_ThreadInfo_t*) pvParameters;
    Assert(NULL != ThreadInfoPtr, ec_NULL);
    Assert(NULL != ThreadInfoPtr->SemaphoreProducer2Consumer, ec_InvalidArgument);
    Assert(NULL != ThreadInfoPtr->QueuePrint,                 ec_InvalidArgument);
    Assert(NULL != ThreadInfoPtr->Lock_Init,                  ec_InvalidArgument);

    ets_ThreadInfo_t ThreadInfo;
    memcpy(&ThreadInfo, pvParameters, sizeof(ets_ThreadInfo_t));
    xSemaphoreGive(ThreadInfo.Lock_Init); // task creator may now reuse *pvParameters

    while (1) {
        et_PrintJob_t PrintJob;
        if ( xQueueReceive(ThreadInfo.QueuePrint, &PrintJob,  (portTickType) -1) ) {
#ifdef EXTENSION_400_lcd_320x240_olimex
            if ( (PrintJob.Column != -1) && (PrintJob.Row != -1) )
                LCD_ConSetPos(PrintJob.Row, PrintJob.Column);
            LCD_Printf(PrintJob.Text, PrintJob.Value);
#endif
        }
    }
}

//} Function definitions
