#!/bin/bash
#
#  Install script for FreeRTOS Multitasking Schedulder for Toolchain_STM32
#  Script written by Gregor Rebel 2010-2011.
# 
#  Feel free do distribute and change to your own needs!
#
# Tested successfully on:
# Ubuntu 10.04 Netbook Edition x86
# KUbuntu 11.04 x86
# openSuSE 11.4 x86
# XUbuntu 11.10 x64

source scripts/installFuncs.sh

if [ "$USER" == "root" ]; then
  echo "$0: ERROR this script should not been run as user root!"
  exit 10
fi

INSTALLPATH="300_scheduler_free_rtos"
dir "$INSTALLPATH"
cd "$INSTALLPATH"

#{ IMPLEMENT YOUR INSTALL SCRIPT BELOW -----------------------------------------------

if [ ! -e OK.Install ]; then #{

  ERROR=""
  
  if [ ! -e OK.Packages ]; then #{
    installPackage libgmp3-dev 
    installPackage libmpfr-dev
    touch OK.Packages
  fi #}
  
  #Version="6.1.1"
  #Version="7.0.2"
  Version="7.1.0"
  Folder="FreeRTOSV${Version}"
  Archive="FreeRTOSV${Version}.zip"
  if [ ! -d $Folder ]; then
    ERROR=""
    unzip -t $Folder.zip || ERROR="1"
    if [ $ERROR == "1" ]; then
      rm 2>/dev/null $Folder.zip
      echo "Downloading FreeRTOS (server sometimes is very slow).."
      getFile http://dfn.dl.sourceforge.net/project/freertos/FreeRTOS/V${Version}/ $Archive
    fi
    ERROR=""
    unzip -t $Archive || ERROR="1"
    if [ "$ERROR" == "" ]; then
      unZIP FreeRTOS $Archive
      mv FreeRTOS $Folder
      ln -s $Folder current
      #add2CleanUp ../cleanup.sh "rm -Rf $INSTALLPATH/$Folder"
    else
      echo "ERROR: Zip Archive is corrupted ($Folder.zip)!"
      exit 11
    fi
  fi
  
  File="current/Demo/Common/drivers/ST/STM32F10xFWLib/inc/stm32f10x_systick.h"
  if [ ! -e "${File}_orig" ]; then #{
      makeOrig ${File}
      echo "modifing ${File}.."
      ls -lh $File
      replaceInFile ${File} '#include "stm32f10x_map.h"' 'ToolChain_STM32 ERROR: Do not include this file! #include "stm32f10x_map.h" '
  fi #}
  
  File="current/Demo/Common/drivers/ST/STM32F10xFWLib/inc/stm32f10x_rcc.h"
  if [ ! -e "${File}_orig" ]; then #{
      makeOrig ${File}
      echo "modifing ${File}.."
      ls -lh $File
      replaceInFile ${File} '#include "stm32f10x_map.h"' '//ToolChain_STM32 ERROR: Do not include this file! #include "stm32f10x_map.h" '
  fi #}
  
  File="current/Source/include/mpu_wrappers.h"
  if [ ! -e "${File}_orig" ]; then #{
      mv ${File} ${File}_orig
      echo "modifing ${File}.."
      sed -e "s/\#define PRIVILEGED_FUNCTION __attribute__\(\(section\(\"privileged_functions\"\)\)\)/\#define PRIVILEGED_FUNCTION \/\/__attribute__\(\(section\(\"privileged_functions\"\)\)\)/g" ${File}_orig >${File}
  fi #}
  
  #X File="current/Demo/Common/drivers/ST/STM32F10xFWLib/inc/stm32f10x_type.h"
  #X if [ ! -e "${File}_orig" ]; then #{
  #X     makeOrig ${File}
  #X     echo "`pwd`> modifing ${File}.."
  #X     Comment="// previous content of this file is already defined in std_peripherals_library\/CMSIS\...\/stm32f10x.h"
  #X     echo "$Comment" >${File}
  #X 
  #X     #X replaceInFile ${File} "typedef signed long  s32;" "// typedef signed long  s32; // $Comment"
  #X     #X replaceInFile ${File} "typedef signed short s16;" "// typedef signed short s16; // $Comment"
  #X     #X replaceInFile ${File} "typedef signed char  s8;"  "// typedef signed char  s8;  // $Comment"
  #X   
  #X     #X replaceInFile ${File} "typedef volatile signed long  vs32;" "// typedef volatile signed long  vs32; // $Comment"
  #X     #X replaceInFile ${File} "typedef volatile signed short vs16;" "// typedef volatile signed short vs16; // $Comment"
  #X     #X replaceInFile ${File} "typedef volatile signed char  vs8;"  "// typedef volatile signed char  vs8; // $Comment"
  #X 
  #X     #X replaceInFile ${File} "typedef unsigned long  u32;"         "// typedef unsigned long  u32; // $Comment"
  #X     #X replaceInFile ${File} "typedef unsigned short u16;"         "// typedef unsigned short u16; // $Comment"
  #X     #X replaceInFile ${File} "typedef unsigned char  u8;"          "// typedef unsigned char  u8; // $Comment"
  #X fi #}
  
  Comment="correct type of usStackDepth improves visibility during gcc debugging"
  File="current/Source/include/task.h"
  if [ ! -e "${File}_orig" ]; then #{
      makeOrig ${File}
      echo "`pwd`> modifing ${File}.. ($Comment)"
  
      replaceInFile ${File} "unsigned short usStackDepth;" "size_t usStackDepth; //ToolChain_STM32 $Comment"
  fi #}
  
  File="current/Source/tasks.c"
  if [ ! -e "${File}_orig" ]; then #{
      makeOrig ${File}
      echo "`pwd`> modifing ${File}.. ($Comment)"
  
      replaceInFile ${File} "unsigned short usStackDepth;" "size_t usStackDepth; //ToolChain_STM32 $Comment"
  fi #}

  Error=""
  if [ ! -e current ]; then
    Error="Missing symbolic link current !"
  fi
  if [ ! -d $Folder ]; then
    Error="Missing folder $Folder !"
  fi
  if [ "$Error" == "" ]; then
    echo "" >OK.Install
  fi
fi #} [ ! -e OK.Install ]; then
if [   -e OK.Install ]; then #{
  Name="${INSTALLPATH}"
  createExtensionMakefileHead ${Name} #{
  cat <<END_OF_MAKEFILE >>../extensions/makefile.$Name #{

FREE_RTOS=1
RT_SCHEDULER=FREE_RTOS
COMPILE_OPTS += -DRT_SCHEDULER=\$(RT_SCHEDULER)

#activate FreeRTOS parts of source code
COMPILE_OPTS += -DFREE_RTOS -DEXTENSION_$Name

# add include directories
INCLUDE_DIRS += -I additionals/$Name/Source/include/ \\
                -I additionals/$Name/Source/portable/GCC/ARM_CM3/

#                -I additionals/$Name/Demo/Common/drivers/ST/STM32F10xFWLib/inc/

# add source directories
VPATH +=  additionals/$Name/Source/ \\
          additionals/$Name/Source/portable/MemMang/ \\
          additionals/$Name/Source/portable/GCC/ARM_CM3/ \\

# disable warnings 
# (W_CONVERSION=-Wconversion produces lots of warnings from FreeRTOS code)
W_CONVERSION=

# redefine FreeRTOS exception handlers --------------------------------------
# in additionals/$Name/Source/portable/GCC/ARM_CM3/port.c to match with CMSIS vector handlers 
# in std_peripherals_library/CMSIS/CM3/DeviceSupport/ST/STM32F10x/startup/gcc_ride7/startup_stm32f10x_md.s
# (see http://www.mikrocontroller.net/topic/148988)
AS_OPTS=-DUSART1_IRQHandler=vUARTInterruptHandler \\
          -DADC1_2_IRQHandler=ADC_IRQHandler \\
          -DSysTick_Handler=xPortSysTickHandler \\
          -DSVC_Handler=vPortSVCHandler \\
          -DPendSV_Handler=xPortPendSVHandler

# add FreeRTOS object-files
MAIN_OBJS += port.o queue.o list.o tasks.o
#? syscalls.o 

END_OF_MAKEFILE
#}
  createExtensionMakefileTail ${Name} #}
  File="../extensions/activate.${Name}.sh"
  createActivateScriptHead $File $0 "Multitasking scheduler FreeRTOS (http://freertos.org)" #{
  echo 'rm 2>/dev/null $Dir_ExtensionsActive/makefile.300_scheduler_*' >>$File

  # create links into extensions.active/
  echo "createLink \$Dir_Extensions/makefile.$Name \$Dir_ExtensionsActive/makefile.$Name '' QUIET" >>$File
  
  createActivateScriptTail $File $Name #}

  Name="${INSTALLPATH}_heap1"
  createExtensionMakefileHead ${Name} #{
  cat <<END_OF_MAKEFILE >>../extensions/makefile.$Name

EXTENSION_heap=1

COMPILE_OPTS += -DSCHEDULER_HEAP=$Name

# can only allocate    (deterministic, fast, minimal memory usage)
MAIN_OBJS += heap_1.o 

END_OF_MAKEFILE
  createExtensionMakefileTail ${Name} #}
  File="../extensions/activate.${Name}.sh"
  createActivateScriptHead $File $0 "memory allocation scheme 1: can only allocate; smallest memory + processing overhead" #{
  echo 'rm 2>/dev/null $Dir_ExtensionsActive/makefile.300_scheduler_*_heap*' >>$File

  # create links into extensions.active/
  echo "createLink \$Dir_Extensions/makefile.$Name \$Dir_ExtensionsActive/makefile.$Name '' QUIET" >>$File
  
  createActivateScriptTail $File $Name #}

  Name="${INSTALLPATH}_heap2"
  createExtensionMakefileHead ${Name} #{
  cat <<END_OF_MAKEFILE >>../extensions/makefile.$Name 

EXTENSION_heap=2

COMPILE_OPTS += -DSCHEDULER_HEAP=$Name

# allocate/ deallocate (deterministic, high memory usage)
MAIN_OBJS += heap_2.o 

END_OF_MAKEFILE
  createExtensionMakefileTail ${Name} #}
  File="../extensions/activate.${Name}.sh"
  createActivateScriptHead $File $0 "memory allocation scheme 2: allocate/ deallocate (deterministic, high memory usage)" #{
  echo 'rm 2>/dev/null $Dir_ExtensionsActive/makefile.300_scheduler_*_heap*' >>$File

  # create links into extensions.active/
  echo "createLink \$Dir_Extensions/makefile.$Name \$Dir_ExtensionsActive/makefile.$Name '' QUIET" >>$File
  
  createActivateScriptTail $File $Name #}

  Name="${INSTALLPATH}_heap3"
  createExtensionMakefileHead ${Name} #{
  cat <<END_OF_MAKEFILE >>../extensions/makefile.$Name

EXTENSION_heap=3

COMPILE_OPTS += -DSCHEDULER_HEAP=$Name

# allocate/ deallocate (full implementation uses malloc()/ free())
MAIN_OBJS += heap_3.o 

END_OF_MAKEFILE
  createExtensionMakefileTail ${Name} #}
  File="../extensions/activate.${Name}.sh"
  createActivateScriptHead $File $0 "memory allocation scheme 3: allocate/ deallocate (full implementation uses malloc()/ free())" #{
  echo 'rm 2>/dev/null $Dir_ExtensionsActive/makefile.300_scheduler_*_heap*' >>$File

  # create links into extensions.active/
  echo "createLink \$Dir_Extensions/makefile.$Name \$Dir_ExtensionsActive/makefile.$Name '' QUIET" >>$File
  
  createActivateScriptTail $File $Name #}
  
  addLine ../scripts/createLinks.sh "rm 2>/dev/null $INSTALLPATH;  ln -sv \$Source/ToolChain_STM32/ToolChain/$INSTALLPATH/current  $INSTALLPATH"
  addLine ../scripts/createLinks.sh "rm 2>/dev/null ${INSTALLPATH}_network_lwip;  ln -sv \$Source/ToolChain_STM32/ToolChain/$INSTALLPATH/current/Demo/Common/ethernet/lwip-1.4.0/  ${INSTALLPATH}_network_lwip"
  echo "$INSTALLPATH installed successfully"
#}
else
  echo "failed to install $INSTALLPATH"
  exit 10
fi

#} IMPLEMENT YOUR INSTALL SCRIPT ABOVE -----------------------------------------------

exit 0
