#!/bin/bash

#
#  Install script for Fixpoint Mathematical Library 
#  URL: http://code.google.com/p/fixpointlib/
#
#  Script written by Gregor Rebel 2010-2011.
# 
#  Feel free do distribute and change to your own needs!
#
# Tested successfully on:
# Ubuntu 10.04 Netbook Edition x86
# KUbuntu 11.04 x86
# openSuSE 11.4 x86
# XUbuntu 11.10 x64

source scripts/installFuncs.sh

if [ "$USER" == "root" ]; then
  echo "$0: ERROR this script should not been run as user root!"
  exit 10
fi

INSTALLPATH="100_board_olimex_h103"
dir "$INSTALLPATH"
cd "$INSTALLPATH"

#{ IMPLEMENT YOUR INSTALL SCRIPT BELOW -----------------------------------------------
  Dir=`pwd`
  if [ ! -e OK.Docs ]; then #{
    Get_Fails=""
    dir ../../Documentation/Boards
    
    getFile http://olimex.com/dev/pdf/ARM/ST/ STM32-H103.pdf Board_STM32-H103.pdf ../../Documentation/Boards/
    
    if [ "$Get_Fails" == "" ]; then
      touch OK.Docs
    else
      echo "$0 - ERROR: missing files: $Get_Fails"
    fi
  fi #}

  Name="${INSTALLPATH}" #{
  createExtensionMakefileHead ${Name} #{
  File="../extensions/makefile.${Name}" 
  cat <<END_OF_MAKEFILE >>$File 

# activate certain parts of source-code
COMPILE_OPTS += -DEXTENSION_${Name}=1
COMPILE_OPTS += -DBOARD=$Name
BOARD=$Name

# define microcontroller used on board
STM32F10X_MD  =1
COMPILE_OPTS += -DUC_DEVICE=STM32F103RBT6
COMPILE_OPTS += -DUC_PACKAGE_PINS=64

COMPILE_OPTS += -DRAM_SIZE_K=20
COMPILE_OPTS += -DRAM_START=0x20000000
COMPILE_OPTS += -DFLASH_SIZE_K=128
COMPILE_OPTS += -DFLASH_START=0x08000000

# Define port pins
# reference: http://olimex.com/dev/images/ARM/ST/STM32-H103-sch.gif

COMPILE_OPTS += -DTTC_LED1=PIN_PC12
COMPILE_OPTS += -DTTC_LED2=PIN_PC12      // using same LED as LED1

COMPILE_OPTS += -DTTC_SWITCH1=PIN_PA0    // press button switch 1 (named BUT)
COMPILE_OPTS += -DTTC_SWITCH1_TYPE=tgm_input_floating # -> ttc_gpio_types.h
COMPILE_OPTS += -DTTC_SWITCH1_LOWACTIVE=1

#{ USART1  - define universal synchronous asynchronous receiver transmitter #1
  COMPILE_OPTS += -DTTC_USART1=USART1       # 
  COMPILE_OPTS += -DTTC_USART1_TX=PIN_PA9   # EXT1-4
  COMPILE_OPTS += -DTTC_USART1_RX=PIN_PA10  # EXT1-7
  
  # Pins for RTS,CTS are also used for USBDM,USBDP!
  #COMPILE_OPTS += -DTTC_USART1_CTS=PIN_PA11 # EXT1-1
  #COMPILE_OPTS += -DTTC_USART1_RTS=PIN_PA12 # EXT1-3
#}USART1
#{ USART2  - define universal synchronous asynchronous receiver transmitter #2
  COMPILE_OPTS += -DTTC_USART2=USART2       # RS232 connector connected USART
  COMPILE_OPTS += -DTTC_USART2_TX=PIN_PA2   # EXT2-7
  COMPILE_OPTS += -DTTC_USART2_RX=PIN_PA3   # EXT2-10
  COMPILE_OPTS += -DTTC_USART2_CTS=PIN_PA0  # EXT2-4
  COMPILE_OPTS += -DTTC_USART2_RTS=PIN_PA1  # EXT2-8
#}USART2
#{ SPI1    - define serial peripheral interface #1
  COMPILE_OPTS += -DTTC_SPI1=SPI1          # 
  COMPILE_OPTS += -DTTC_SPI1_MOSI=PIN_PA7  # 
  COMPILE_OPTS += -DTTC_SPI1_MISO=PIN_PA6  # 
  COMPILE_OPTS += -DTTC_SPI1_SCK=PIN_PA5   # 
  COMPILE_OPTS += -DTTC_SPI1_NSS=PIN_PA4   # 
#}SPI1
#{ I2C1    - define inter-integrated circuit interface #1
  COMPILE_OPTS += -DTTC_I2C1=I2C1           # internal device being used for this port
  COMPILE_OPTS += -DTTC_I2C1_SDA=PIN_PB7    # EXT1-15
  COMPILE_OPTS += -DTTC_I2C1_SCL=PIN_PB6    # EXT1-13
  COMPILE_OPTS += -DTTC_I2C1_SMBAL=PIN_PB5  # EXT1-12
#}I2C1
#{ I2C2    - define inter-integrated circuit interface #2
  COMPILE_OPTS += -DTTC_I2C2=I2C2           # internal device being used for this port
  COMPILE_OPTS += -DTTC_I2C2_SDA=PIN_PB11   # EXT2-15
  COMPILE_OPTS += -DTTC_I2C2_SCL=PIN_PB10   # EXT2-14
  COMPILE_OPTS += -DTTC_I2C2_SMBAL=PIN_PB12 # EXT2-17
#}I2C2

END_OF_MAKEFILE
  createExtensionMakefileTail ${Name} #}
  File="../extensions/activate.${Name}.sh" #{
  createActivateScriptHead $File $0 "Protoboard H103 from Olimex -> http://olimex.com/dev/stm32-h103.html"
  cat <<END_OF_ACTIVATE >>$File #{

  rm 2>/dev/null \$Dir_ExtensionsActive/makefile.100_board_*
  
  # create links into extensions.active/
  createLink \$Dir_Extensions/makefile.$Name \$Dir_ExtensionsActive/makefile.$Name '' QUIET
  
  activate.200_cpu_stm32f10x.sh QUIET "\$0"

END_OF_ACTIVATE
#}
  createActivateScriptTail $File $Name
  #}

#}
  echo "Installed successfully: $INSTALLPATH"

cd ..
rmdir 2>/dev/null $INSTALLPATH
#} IMPLEMENT YOUR INSTALL SCRIPT ABOVE -----------------------------------------------

exit 0
