#!/bin/bash

#
#  Install script for Fixpoint Mathematical Library 
#  URL: http://code.google.com/p/fixpointlib/
#
#  Script written by Gregor Rebel 2010-2011.
# 
#  Feel free do distribute and change to your own needs!
#
# Tested successfully on:
# Ubuntu 10.04 Netbook Edition x86
# KUbuntu 11.04 x86
# openSuSE 11.4 x86
# XUbuntu 11.10 x64

source scripts/installFuncs.sh

if [ "$USER" == "root" ]; then
  echo "$0: ERROR this script should not been run as user root!"
  exit 10
fi

INSTALLPATH="100_board_olimex_lcd"
dir "$INSTALLPATH"
cd "$INSTALLPATH"

#{ IMPLEMENT YOUR INSTALL SCRIPT BELOW -----------------------------------------------
  Dir=`pwd`
  if [ ! -e $Dir/OK.Docs ]; then #{
    Get_Fails=""
    dir ../../Documentation/Boards
    cd ../../Documentation/Boards
    getFile http://olimex.com/dev/pdf/ARM/ST/ STM32-LCD.pdf Board_STM32-LCD.pdf
    getFile http://www.olimex.com/dev/pdf/PIC/ "FS-K320QVB-V1-01-3T%20(3).pdf" LCD_FS-K320QVB-V1-01-3T.pdf
    getFile http://olimex.com/dev/ stm32-lcd.html
    if [ "$Get_Fails" == "" ]; then
      touch $Dir/OK.Docs
    else
      echo "$0 - ERROR: missing files: $Get_Fails"
    fi
  fi #}
  cd "$Dir"

  Name="${INSTALLPATH}" #{
  createExtensionMakefileHead ${Name} #{
  File="../extensions/makefile.${Name}" 
  cat <<END_OF_MAKEFILE >>$File 

# activate certain parts of source-code
COMPILE_OPTS += -DEXTENSION_${Name}=1
COMPILE_OPTS += -DBOARD=$Name
BOARD=$Name

# define microcontroller used on board
STM32F10X_HD  =1
COMPILE_OPTS += -DUC_DEVICE=STM32F103ZE
COMPILE_OPTS += -DUC_PACKAGE_PINS=144

COMPILE_OPTS += -DRAM_SIZE_K=64
COMPILE_OPTS += -DRAM_START=0x20000000
COMPILE_OPTS += -DFLASH_SIZE_K=512
COMPILE_OPTS += -DFLASH_START=0x08000000

# Define port pins
# reference: http://olimex.com/dev/ARM/ST/STM32-LCD/STM32-LCD-schematic.pdf

#{ USART1  - define universal synchronous asynchronous receiver transmitter #1
COMPILE_OPTS += -DTTC_USART1=USART1       # RS232 on 10-pin male header connector UEXT1 connected USART
COMPILE_OPTS += -DTTC_USART1_TX=PIN_PA9   # UEXT1-3, UEXT40-3
COMPILE_OPTS += -DTTC_USART1_RX=PIN_PA10  # UEXT1-4, UEXT40-4
COMPILE_OPTS += -DTTC_USART1_CK=PIN_PA8   # EXT-40
#}USART1
#{ USART2  - define universal synchronous asynchronous receiver transmitter #2
  COMPILE_OPTS += -DTTC_USART2=USART2      # RS232 on 10-pin male header connector UEXT2 connected USART
  COMPILE_OPTS += -DTTC_USART2_TX=PIN_PA2  # UEXT2-3
  COMPILE_OPTS += -DTTC_USART2_RX=PIN_PA3  # UEXT2-4
  COMPILE_OPTS += -DTTC_USART2_CK=PIN_PA4  # UEXT1-10
#}USART2
#{ SPI1    - define serial peripheral interface #1
  COMPILE_OPTS += -DTTC_SPI1=SPI1          # 
  COMPILE_OPTS += -DTTC_SPI1_MOSI=PIN_PA7  # UEXT1-8,  UEXT40-8
  COMPILE_OPTS += -DTTC_SPI1_MISO=PIN_PA6  # UEXT1-7,  UEXT40-7
  COMPILE_OPTS += -DTTC_SPI1_SCK=PIN_PA5   # UEXT1-9,  UEXT40-9
  COMPILE_OPTS += -DTTC_SPI1_NSS=PIN_PA4   # UEXT1-10, UEXT40-10
#}SPI1
#{ I2C1    - define inter-integrated circuit interface #1
  COMPILE_OPTS += -DTTC_I2C1=I2C1           # internal device being used for this port
  COMPILE_OPTS += -DTTC_I2C1_SDA=PIN_PB7    # UEXT2-5/ UEXT40-6
  COMPILE_OPTS += -DTTC_I2C1_SCL=PIN_PB6    # UEXT2-6/ UEXT40-5
  COMPILE_OPTS += -DTTC_I2C1_SMBAL=PIN_PB5  # EXT-10
#}I2C1
#{ I2C2    - define inter-integrated circuit interface #2
  COMPILE_OPTS += -DTTC_I2C2=I2C2           # internal device being used for this port
  COMPILE_OPTS += -DTTC_I2C2_SDA=PIN_PB11   # connected to accelerometer
  COMPILE_OPTS += -DTTC_I2C2_SCL=PIN_PB10   # connected to accelerometer
#}I2C2

END_OF_MAKEFILE
  createExtensionMakefileTail ${Name} #}
  File="../extensions/activate.${Name}.sh" #{
  createActivateScriptHead $File $0 "Protoboard LCD from Olimex -> http://olimex.com/dev/stm32-lcd.html"
  cat <<END_OF_ACTIVATE >>$File #{

  rm 2>/dev/null \$Dir_ExtensionsActive/makefile.100_board_*
  
  # create links into extensions.active/
  createLink \$Dir_Extensions/makefile.$Name \$Dir_ExtensionsActive/makefile.$Name '' QUIET
  
  activate.200_cpu_stm32f10x.sh QUIET "\$0"
  activate.400_lcd_320x240_olimex.sh QUIET "\$0"


END_OF_ACTIVATE
#}
  createActivateScriptTail $File $Name
  #}

#}
  echo "Installed successfully: $INSTALLPATH"

cd ..
rmdir 2>/dev/null $INSTALLPATH
#} IMPLEMENT YOUR INSTALL SCRIPT ABOVE -----------------------------------------------

exit 0
