#!/bin/bash

#
#  Install script for Fixpoint Mathematical Library 
#  URL: http://code.google.com/p/fixpointlib/
#
#  Script written by Gregor Rebel 2010-2011.
# 
#  Feel free do distribute and change to your own needs!
#
# Tested successfully on:
# Ubuntu 10.04 Netbook Edition x86
# KUbuntu 11.04 x86
# openSuSE 11.4 x86
# XUbuntu 11.10 x64

source scripts/installFuncs.sh

if [ "$USER" == "root" ]; then
  echo "$0: ERROR this script should not been run as user root!"
  exit 10
fi

INSTALLPATH="100_board_wsn3_stm32w108"
dir "$INSTALLPATH"
cd "$INSTALLPATH"

#{ IMPLEMENT YOUR INSTALL SCRIPT BELOW -----------------------------------------------
  Dir=`pwd`
  if [ ! -e $Dir/OK.Docs ]; then #{
    Get_Fails=""
    dir ../../Documentation/Boards
    cd ../../Documentation/Boards
    getFile http://olimex.com/dev/pdf/ARM/ST/ STM32-LCD.pdf Board_STM32-LCD.pdf
    getFile http://www.olimex.com/dev/pdf/PIC/ "FS-K320QVB-V1-01-3T%20(3).pdf" LCD_FS-K320QVB-V1-01-3T.pdf
    getFile http://olimex.com/dev/ stm32-lcd.html
    if [ "$Get_Fails" == "" ]; then
      touch $Dir/OK.Docs
    else
      echo "$0 - ERROR: missing files: $Get_Fails"
    fi
  fi #}
  cd "$Dir"

  Name="${INSTALLPATH}" #{
  createExtensionMakefileHead ${Name} #{
  File="../extensions/makefile.${Name}" 
  cat <<END_OF_MAKEFILE >>$File 

# activate certain parts of source-code
COMPILE_OPTS += -DEXTENSION_${Name}=1
COMPILE_OPTS += -DBOARD=$Name
BOARD=$Name

# uController to use
STM32W108   = 1

COMPILE_OPTS += -DuCONTROLLER=STM32W108

# Note: Make sure to set correct sizes for RAM and Flash
# Example for stm32f10x
COMPILE_OPTS += -DRAM_SIZE_K=8
COMPILE_OPTS += -DRAM_START=0x20000000    # all stm32 uCs have same start address  
COMPILE_OPTS += -DFLASH_SIZE_K=128
COMPILE_OPTS += -DFLASH_START=0x08000000  # all stm32 uCs have same start address

# add required obbject-files
MAIN_OBJS += adc.o
MAIN_OBJS += temperature_sensor.o
MAIN_OBJS += system-timer.o
MAIN_OBJS += board.o
MAIN_OBJS += uart.o
MAIN_OBJS += mfg-token.o
MAIN_OBJS += sleep.o
MAIN_OBJS += clocks.o
MAIN_OBJS += micro-common.o
MAIN_OBJS += led.o
MAIN_OBJS += micro.o
MAIN_OBJS += nvm.o
MAIN_OBJS += flash.o
MAIN_OBJS += mems.o
MAIN_OBJS += button.o
MAIN_OBJS += micro-common-internal.o

# set some well known port defines
# Note: Using the Constant names listed here makes it easier to reuse example code! 

# LED-pins (push pull output)
COMPILE_OPTS += -DTTC_LED1=PIN_PC6
COMPILE_OPTS += -DTTC_LED2=PIN_PC7

BoardRev10=1
#BoardRev11=1

# Switches (input with pull up resistor)
COMPILE_OPTS += -DTTC_SWITCH1=PIN_PB0   
COMPILE_OPTS += -DTTC_SWITCH1_TYPE=tgm_input_floating # -> ttc_gpio_types.h
COMPILE_OPTS += -DTTC_SWITCH2=PIN_PB6
COMPILE_OPTS += -DTTC_SWITCH2_TYPE=tgm_input_floating # -> ttc_gpio_types.h

ifdef BoardRev10  #{
  COMPILE_OPTS += -DTTC_SWITCH3=PIN_PA1
  COMPILE_OPTS += -DTTC_SWITCH3_TYPE=tgm_input_floating # -> ttc_gpio_types.h
  COMPILE_OPTS += -DTTC_SWITCH4=PIN_PA2
  COMPILE_OPTS += -DTTC_SWITCH4_TYPE=tgm_input_floating # -> ttc_gpio_types.h
endif
#}

#{ USART1 - define serial port #1
  COMPILE_OPTS += -DTTC_USART1=UART1
  COMPILE_OPTS += -DTTC_USART1_TX=PIN_PB1
  COMPILE_OPTS += -DTTC_USART1_RX=PIN_PB2
  COMPILE_OPTS += -DTTC_USART1_RTS=PIN_PB4 
  COMPILE_OPTS += -DTTC_USART1_CTS=PIN_PB3
#}USART1
#{ SPI1   - define serial peripheral interface port #1
  COMPILE_OPTS += -DTTC_SPI1=SPI1           # internal device being used for this port
  COMPILE_OPTS += -DTTC_SPI1_MOSI=PIN_PB1
  COMPILE_OPTS += -DTTC_SPI1_MISO=PIN_PB2
  COMPILE_OPTS += -DTTC_SPI1_SCK=PIN_PB3
  COMPILE_OPTS += -DTTC_SPI1_NSS=PIN_PB4
#}SPI1
#{ SPI2   - define serial peripheral interface port #2
  COMPILE_OPTS += -DTTC_SPI2=SPI2
  COMPILE_OPTS += -DTTC_SPI2_MOSI=PIN_PA0
  COMPILE_OPTS += -DTTC_SPI2_MISO=PIN_PA1
  COMPILE_OPTS += -DTTC_SPI2_SCK=PIN_PA2
  COMPILE_OPTS += -DTTC_SPI2_NSS=PIN_PA3
#}SPI2
#{ I2C1   - (Inter-Integrated Circuit-) Interface #1
  COMPILE_OPTS += -DTTC_I2C1=I2C1
  COMPILE_OPTS += -DTTC_I2C1_SDA=PIN_PB1  # CONN_30-16
  COMPILE_OPTS += -DTTC_I2C1_SCL=PIN_PB2  # CONN_30-15
#}I2C1
#{ I2C2   - (Inter-Integrated Circuit-) Interface #2
  COMPILE_OPTS += -DTTC_I2C2=I2C2
  COMPILE_OPTS += -DTTC_I2C2_SDA=PIN_PA1  # CONN_30-22
  COMPILE_OPTS += -DTTC_I2C2_SCL=PIN_PA2  # CONN_30-21
#}I2C1

END_OF_MAKEFILE
  createExtensionMakefileTail ${Name} #}
  File="../extensions/activate.${Name}.sh" #{
  createActivateScriptHead $File $0 "Protoboard LCD from Olimex -> http://olimex.com/dev/stm32-lcd.html"
  cat <<END_OF_ACTIVATE >>$File #{

  rm 2>/dev/null \$Dir_ExtensionsActive/makefile.100_board_*
  
  # create links into extensions.active/
  createLink \$Dir_Extensions/makefile.$Name \$Dir_ExtensionsActive/makefile.$Name '' QUIET
  
  activate.200_cpu_stm32w1xx.sh QUIET "\$0"
  activate.200_cpu_stm32w1xx_hal.sh QUIET "\$0"


END_OF_ACTIVATE
#}
  createActivateScriptTail $File $Name
  #}

#}
  echo "Installed successfully: $INSTALLPATH"

cd ..
rmdir 2>/dev/null $INSTALLPATH
#} IMPLEMENT YOUR INSTALL SCRIPT ABOVE -----------------------------------------------

exit 0
