#!/bin/bash

#
#  Install script for combined Microcontroller + 2,4GHz radio STM32 Wxxx 
#  with ARM Cortex M3 core.
#
#  Script written by Gregor Rebel 2012
# 
#  Feel free do distribute and change to your own needs!
#
# Tested successfully on:
# openSuSE 12.1 x86
# XUbuntu 11.10 x64

source scripts/installFuncs.sh

if [ "$USER" == "root" ]; then
  echo "$0: ERROR this script should not been run as user root!"
  exit 10
fi

Rank="200"
INSTALL_PREFIX="cpu_stm32w1xx"
INSTALLPATH="${Rank}_$INSTALL_PREFIX"
dir "$INSTALLPATH"
cd "$INSTALLPATH"

#{ IMPLEMENT YOUR INSTALL SCRIPT BELOW -----------------------------------------------
  
  if [ ! -e OK.Packages ]; then #{      install required software packages
    installPackage wine 
    touch OK.Packages
  fi #}
  if [ ! -e OK.Documentation ]; then #{ download documentation
    Dir=`pwd`
    Get_Fails=""
    dir ../../Documentation/uC
    dir ../../Documentation/uC/STM32W1xx
    
    Error=""
    #{ download docs..
    getFile http://www.st.com/internet/com/TECHNICAL_RESOURCES/TECHNICAL_LITERATURE/PROGRAMMING_MANUAL/ CD00280769.pdf PM0073-STM32W1xx_Programming_Manual.pdf                                         ../../Documentation/uC/STM32W1xx/
    getFile http://www.st.com/internet/com/TECHNICAL_RESOURCES/TECHNICAL_LITERATURE/USER_MANUAL/        CD00262415.pdf UM0894-STM32W-SK_and_STM32W-EXT_starter_and_extension_kits.pdf                  ../../Documentation/uC/STM32W1xx/
    getFile http://www.st.com/internet/com/TECHNICAL_RESOURCES/TECHNICAL_LITERATURE/USER_MANUAL/        CD00280375.pdf UM0978-Using_the_Simple_MAC_nodetest_application.pdf                            ../../Documentation/uC/STM32W1xx/
    getFile http://www.st.com/internet/com/TECHNICAL_RESOURCES/TECHNICAL_LITERATURE/USER_MANUAL/        CD00264812.pdf UM0909-STM32W108xx_ZigBee_RF4CE_library.pdf                                     ../../Documentation/uC/STM32W1xx/
    #Not available anymore: getFile http://www.st.com/internet/com/TECHNICAL_RESOURCES/TECHNICAL_LITERATURE/APPLICATION_NOTE/   CD00271698.pdf AN3211-Using_the_simulated_EEPROM_with_the_STM32W108_platform.pdf               ../../Documentation/uC/STM32W1xx/
    getFile http://www.po-star.com/public/uploads/                                              20120227121858_397.pdf AN3211-Using_the_simulated_EEPROM_with_the_STM32W108_platform.pdf               ../../Documentation/uC/STM32W1xx/
    getFile http://www.st.com/internet/com/TECHNICAL_RESOURCES/TECHNICAL_LITERATURE/APPLICATION_NOTE/   CD00282958.pdf AN3262-Using_the_over-the-air_bootloader_with_STM32W108_devices.pdf             ../../Documentation/uC/STM32W1xx/
    #Not available anymore: getFile http://www.st.com/internet/com/TECHNICAL_RESOURCES/TECHNICAL_LITERATURE/APPLICATION_NOTE/   CD00269334.pdf AN3188-Preparing_custom_devices_for_the_STM32W108_platform.pdf                  ../../Documentation/uC/STM32W1xx/
    getFile http://www.st.com/internet/com/TECHNICAL_RESOURCES/TECHNICAL_LITERATURE/APPLICATION_NOTE/   CD00270908.pdf AN3206-PCB_design_guidelines_for_the_STM32W108_platform.                        ../../Documentation/uC/STM32W1xx/
    getFile http://www.st.com/internet/com/TECHNICAL_RESOURCES/TECHNICAL_LITERATURE/APPLICATION_NOTE/   DM00024648.pdf AN3359-Low_cost_PCB_antenna_for_2.4GHz_radio_Meander_design.pdf                 ../../Documentation/uC/STM32W1xx/
    getFile http://www.st.com/internet/com/TECHNICAL_RESOURCES/TECHNICAL_LITERATURE/APPLICATION_NOTE/   CD00004479.pdf AN1709-EMC_design_guide_for_ST_microcontrollers.pdf                             ../../Documentation/uC/STM32W1xx/
    getFile http://www.st.com/internet/com/TECHNICAL_RESOURCES/TECHNICAL_LITERATURE/APPLICATION_NOTE/   CD00274443.pdf AN3218-Adjacent_channel_rejection_measurements_for_the_STM32W108_platform.pdf   ../../Documentation/uC/STM32W1xx/
    getFile http://www.st.com/internet/com/TECHNICAL_RESOURCES/TECHNICAL_LITERATURE/DATA_BRIEF/         CD00237927.pdf DB0837-High-performance_802.15.4_wireless_system-on-chip_databrief.pdf          ../../Documentation/uC/STM32W1xx/
    getFile http://www.st.com/internet/com/TECHNICAL_RESOURCES/TECHNICAL_LITERATURE/DATASHEET/          CD00248316.pdf DS6473-High-performance_IEEE_802.15.4_wireless_system-on-chip_Datasheet.pdf     ../../Documentation/uC/STM32W1xx/
    getFile http://www.st.com/internet/com/SALES_AND_MARKETING_RESOURCES/MARKETING_PRESENTATIONS/PRODUCT_PRESENTATION/ stm32w_marketing_pres.pdf  STM32W-Product_presentation.pdf                      ../../Documentation/uC/STM32W1xx/
#}

    if [ "$Get_Fails" == "" ]; then
      echo "Documentation complete: $INSTALLPATH."
      touch OK.Documentation
    else
      echo "ERROR: Documentations not being received: $Get_Fails"
    fi
  fi #}
  if [ ! -e OK.Install ]; then #{       download + install library
    Folder="$HOME/.wine/drive_c/Program Files (x86)/STMicroelectronics"
    if [ ! -d "$Folder" ]; then #{ download + extract archive + run microsoft installer to obtain SimpleMAC library
      echo "missing folder '$Folder'"
      File="STM32W108_SimpleMAC_firmware"
      getFile http://www.st.com/internet/com/SOFTWARE_RESOURCES/SW_COMPONENT/FIRMWARE/ st_zigbee_simplemac.zip $File.zip
      unZIP "e" "${File}.zip"
      echo "Starting windows installer for STM32Wxxx SimpleMAC-library. Please install into default directory (press OK in every dialog)!"
      Archive=`ls *SimpleMAC*exe`
      ln -s "$Archive" SimpleMAC.exe
      echo "wine ./SimpleMAC.exe"
    fi #}
    if [ -d "$Folder" ]; then   #{ create links to libraries
      Version=`ls "$Folder/"`
      createLink "$Folder/$Version" .
      createLink "$Version" SimpleMac-Library
      touch OK.Install
    #}
    else  
      echo "$0 - ERROR: Cannot find installed SimpleMAC-library in $Folder !"
    fi
  fi #}
  if [   -e OK.Install ]; then #{       create activate-scripts + makefiles
    ConfigFile="../999_open_ocd/share/openocd/scripts/target/stm32w1xx.cfg"
    if [ ! -e "$ConfigFile" ]; then
      echo "creating cfg-file '`pwd`/$ConfigFile'.."
      cat <<'END_OF_CONFIG' >$ConfigFile #{
# script for stm32w108

if { [info exists CHIPNAME] } {
   set  _CHIPNAME $CHIPNAME
} else {
   set  _CHIPNAME em357
}

if { [info exists ENDIAN] } {
   set  _ENDIAN $ENDIAN
} else {
   set  _ENDIAN little
}

# Work-area is a space in RAM used for flash programming
# By default use 16kB
if { [info exists WORKAREASIZE] } {
   set  _WORKAREASIZE $WORKAREASIZE
} else {
   #set  _WORKAREASIZE 0x4000
   set  _WORKAREASIZE 0x2000
}

# JTAG speed should be <= F_CPU/6. F_CPU after reset is 8MHz, so use F_JTAG = 1MHz
adapter_khz 10

adapter_nsrst_delay 100
jtag_ntrst_delay 100

#verify_ircapture disable 
#verify_jtag disable


#jtag scan chain
if { [info exists CPUTAPID ] } {
   set _CPUTAPID $CPUTAPID
} else {
  # See STM Document RM0008
  # Section 26.6.3
   set _CPUTAPID 0x3ba00477
}
jtag newtap $_CHIPNAME cpu -irlen 4 -ircapture 0x1 -irmask 0xf -expected-id $_CPUTAPID

if { [info exists BSTAPID ] } {
   # FIXME this never gets used to override defaults...
   set _BSTAPID $BSTAPID
} else {
  # See STM Document RM0008
  # Section 29.6.2
  # Low density devices, Rev A
  #set _BSTAPID1 0x06412041
  # Medium density devices, Rev A
  #set _BSTAPID2 0x06410041
  # Medium density devices, Rev B and Rev Z
  #set _BSTAPID3 0x16410041
  #set _BSTAPID4 0x06420041
  # High density devices, Rev A
  #set _BSTAPID5 0x06414041
  #set _BSTAPID5 0x06418041
  # Connectivity line devices, Rev A and Rev Z
  #set _BSTAPID6 0x06418041
  # XL line devices, Rev A
  #set _BSTAPID7 0x06430041
  
  
  set _BSTAPID1 0x269a862b
 set _BSTAPID 0x269a862b

}

jtag newtap $_CHIPNAME bs -irlen 4 -ircapture 0x0e -irmask 0xf  -expected-id $_BSTAPID

	#	jtag newtap $_CHIPNAME bs -irlen 5 -expected-id $_BSTAPID1 \
#	-expected-id $_BSTAPID2 -expected-id $_BSTAPID3 \
#	-expected-id $_BSTAPID4 -expected-id $_BSTAPID5 \
#	-expected-id $_BSTAPID6 -expected-id $_BSTAPID7



set _TARGETNAME $_CHIPNAME.cpu
#$_TARGETNAME configure -work-area-phys 0x20000000 -work-area-size $_WORKAREASIZE -work-area-backup 0

target create $_TARGETNAME cortex_m3 -endian $_ENDIAN -chain-position $_TARGETNAME

$_TARGETNAME configure -work-area-phys 0x20000000 -work-area-size $_WORKAREASIZE -work-area-backup 0

# flash size will be probed
set _FLASHNAME $_CHIPNAME.flash
#flash bank $_FLASHNAME stm32f1x 0x08000000 0 0 0 $_TARGETNAME
flash bank $_FLASHNAME em357 0x08000000 0 0 0 $_TARGETNAME

# if srst is not fitted use SYSRESETREQ to
# perform a soft reset
cortex_m3 reset_config sysresetreq

END_OF_CONFIG
#}
    else
      echo "cfg-file already exists: '`pwd`/$ConfigFile'.."
    fi

    ConfigFile="../999_open_ocd/share/openocd/scripts/target/stm32w1xx_flash.cfg"
    if [ ! -e "$ConfigFile" ]; then
      echo "creating cfg-file '`pwd`/$ConfigFile'.."
      cat <<'END_OF_CONFIG' >$ConfigFile #{
# flash script for stm32w108


# finish configuration stage + proceed to run stage
init

# hardware reset processor with halt
soft_reset_halt

# check target state em357 
poll

# list all found falsh banks
flash banks

# identify the flash
flash probe 0

# unprotect flash for writing
#DEPRECATED flash protect_check 0 0

em357 unlock 0
# erasing all flash
#DEPRECATED stm32x mass_erase 0
em357 mass_erase 0

# write file to flash number 0
flash write_bank 0 main.bin 0

# hardware processor reset with run
reset run

#shutdown openocd daemon
shutdown

END_OF_CONFIG
#}
    else
      echo "cfg-file already exists: '`pwd`/$ConfigFile'.."
    fi

    Name="${INSTALLPATH}" #{            makefile + activate-script for CPU STM32W1xx
    createExtensionMakefileHead ${Name} #{
    File="../extensions/makefile.${Name}" 
    cat <<END_OF_MAKEFILE >>$File #{

ARM_CPU     =arm7tdmi
TARGET      =-mcpu=cortex-m3 -mthumb
TARGET_ARCHITECTURE_CM3=1
TARGET_ARCHITECTURE_STM32W=1
TARGET_ARCHITECTURE_PREFIX=STM32
COMPILE_OPTS += -DTARGET_ARCHITECTURE_PREFIX=STM32
COMPILE_OPTS += -DTARGET_ARCHITECTURE_STM32W
COMPILE_OPTS += -DTARGET_ARCHITECTURE_CM3
COMPILE_OPTS += -DTARGET_DATA_WIDTH=32


# list of available devices (activate only one!)
STM32W108   =1       # STM32W Device

# check selected uC + add corresponding settings
ifdef STM32W108
  ifdef uCONTROLLER
    ERROR: STM32W108=1 - Only one uController may be selected!
  endif
  uCONTROLLER=STM32W108
  COMPILE_OPTS += -DuCONTROLLER=STM32W108
  COMPILE_OPTS += -DSTM32W108
  MAIN_OBJS += crt_stm32w108.o context-switch.o spmr.o
endif


# some settings which may be required by HAL or SimpleMAC
COMPILE_OPTS += -DCORTEXM3_STM32W108
COMPILE_OPTS += -DPHY_STM32W108XX -DDISABLE_WATCHDOG -DCORTEXM3_STM32W108xB -DCORTEXM3

# define linker scripts to use
LDFLAGS+= -Tadditionals/200_cpu_stm32w1xx_hal_library/micro/cortexm3/stm32w108/gnu-stm32w108xB.ld
LDFLAGS+= -Tadditionals/200_cpu_stm32w1xx_hal_library/micro/cortexm3/stm32w108/gnu-stm32w108.ld

# define rules for assembler-files which may be compiled into objects
context-switch.o: context-switch.s
spmr.o: spmr.s


END_OF_MAKEFILE
#}
    createExtensionMakefileTail ${Name} #}
    File="../extensions/activate.${Name}.sh" #{
    createActivateScriptHead $File $0 "CPU STM32W1xx"
    cat <<END_OF_ACTIVATE >>$File #{

# create links into extensions.active/
createLink ../extensions/makefile.$Name \$Dir_ExtensionsActive/makefile.$Name '' QUIET

# create link to target script being required by openocd to initialize target cpu
cd \$Dir_Configs
createLink ../additionals/999_open_ocd/target/stm32w1xx.cfg openocd_target.cfg
createLink ../additionals/999_open_ocd/target/stm32w1xx_flash.cfg openocd_flash.cfg
cd ..

cd "$Dir_Additionals"
rm -fv 200_cpu_*
createLink \$Source/ToolChain_STM32/ToolChain/$INSTALLPATH/SimpleMac-Library/  $INSTALLPATH
createLink $INSTALLPATH/STM32W108/hal/                                         ${INSTALLPATH}_hal_library
createLink $INSTALLPATH/STM32W108/simplemac/                                   ${INSTALLPATH}_SimpleMAC_library

cd ..
END_OF_ACTIVATE
#}
    createActivateScriptTail $File $Name
    #}
    #}
    Name="${Rank}_${INSTALL_PREFIX}_hal" #{        makefile + activate-script
    createExtensionMakefileHead ${Name} #{
    File="../extensions/makefile.${Name}" 
    cat <<END_OF_MAKEFILE >>$File #{

COMPILE_OPTS += -DHAL_STANDALONE   ##ifdef HAL_STANDALONE
                                   # This is a simplified version of the full stCalibrateVref
                                   # It is meant to be used on build for hal only application not linked with
                                   # simplemac library. The full version is embedded in the simplemac library.
                                   # The simplified version does not support calibration when manufacturing tokens are missing
                                   # or radio boost mode is activated
                                   # File: micro-common-internal.c

INCLUDE_DIRS += -I additionals/200_cpu_stm32w1xx_hal_library/simplemac/include \
                -I additionals/200_cpu_stm32w1xx_hal_library/micro/ \
                -I additionals/200_cpu_stm32w1xx_hal_library/ \
                -I additionals/200_cpu_stm32w1xx_hal_library/micro/cortexm3/stm32w108/ \
                -I additionals/200_cpu_stm32w1xx_hal_library/micro/cortexm3/ \
                -I additionals/200_cpu_stm32w1xx_hal_library/micro/cortexm3/compiler/ \
                -I additionals/200_cpu_stm32w1xx_hal_library/micro/generic/compiler \
                -I additionals/200_cpu_stm32w1xx_hal_library/micro/cortexm3/compiler \
                -I additionals/200_cpu_stm32w1xx_hal_library/simplemac/include

#? vpath additionals/200_cpu_stm32w1xx_hal_library/micro/cortexm3/stm32w108/

# where to find files of different types
INCLUDE_DIRS += -Iadditionals/200_cpu_stm32w1xx_hal_library/micro/cortexm3/stm32w108/
vpath %.c additionals/200_cpu_stm32w1xx_hal_library/micro/cortexm3/
vpath %.c additionals/200_cpu_stm32w1xx_hal_library/micro/
vpath %.c additionals/200_cpu_stm32w1xx_hal_library/micro/cortexm3/stm32w108/
vpath %s additionals/200_cpu_stm32w1xx_hal_library/micro/cortexm3/stm32w108/
vpath %s additionals/200_cpu_stm32w1xx_hal_library/micro/cortexm3/

END_OF_MAKEFILE
#}
    createExtensionMakefileTail ${Name} #}
    File="../extensions/activate.${Name}.sh" #{
    createActivateScriptHead $File $0 "STM32W1xx Hardware Abstraction Layer library (HAL)"
    cat <<END_OF_ACTIVATE >>$File #{

# create links into extensions.active/
createLink ../extensions/makefile.$Name \$Dir_ExtensionsActive/makefile.$Name '' QUIET

activate.200_cpu_stm32w1xx.sh  QUIET "\$0"

END_OF_ACTIVATE
#}
    createActivateScriptTail $File $Name
    #}
    #}_hal
    Name="${Rank}_${INSTALL_PREFIX}_simple_mac" #{ makefile + activate-script
    createExtensionMakefileHead ${Name} #{
    File="../extensions/makefile.${Name}" 
    cat <<END_OF_MAKEFILE >>$File #{

INCLUDE_DIRS += -I additionals/261_cpu_stm32w1xx_SimpleMAC_library/


vpath %.c additionals/200_cpu_stm32w1xx_hal_library/micro/cortexm3/
vpath %.c additionals/200_cpu_stm32w1xx_hal_library/micro/

# SimpleMAC library is only available as precompiled binary
LD_LIBS += -mthumb additionals/261_cpu_stm32w1xx_SimpleMAC_library/library/libsimplemac-library-gnu.a
LD_LIBS += -L"additionals/ST_SimpleMAC/STM32W108/simplemac/library"

END_OF_MAKEFILE
#}
    createExtensionMakefileTail ${Name} #}
    File="../extensions/activate.${Name}.sh" #{
    createActivateScriptHead $File $0 "STM32W1xx Simple MAC Library - provides access to on-chip transceiver"
    cat <<END_OF_ACTIVATE >>$File #{

# create links into extensions.active/
createLink ../extensions/makefile.$Name \$Dir_ExtensionsActive/makefile.$Name '' QUIET

activate.200_cpu_stm32w1xx.sh

END_OF_ACTIVATE
#}
    createActivateScriptTail $File $Name
    #}
    #}_simple_mac
    
    # create permanent symbolic links to source-directories
    addLine ../scripts/createLinks.sh "rm 2>/dev/null $INSTALLPATH;                      ln -sv \$Source/ToolChain_STM32/ToolChain/$INSTALLPATH/SimpleMac-Library/  $INSTALLPATH"
    addLine ../scripts/createLinks.sh "rm 2>/dev/null ${INSTALLPATH}_hal_library;        ln -sv $INSTALLPATH/STM32W108/hal/                                         ${INSTALLPATH}_hal_library"
    addLine ../scripts/createLinks.sh "rm 2>/dev/null ${INSTALLPATH}_SimpleMAC_library;  ln -sv $INSTALLPATH/STM32W108/simplemac/                                   ${INSTALLPATH}_SimpleMAC_library"

    echo "Installed successfully: $INSTALLPATH"
  fi #}
  cd ..
  rmdir 2>/dev/null $INSTALLPATH  # removes only empty install dir
#} IMPLEMENT YOUR INSTALL SCRIPT ABOVE -----------------------------------------------

exit 0
