#!/bin/bash

#
#  Install script for Color LCD Display 320x240 with Touch screen 
#  as installed on Olimex prototype board STM32-LCD.
#  URL: http://olimex.com/dev/stm32-lcd.html
#
#  Script written by Gregor Rebel 2010-2011.
# 
#  Feel free do distribute and change to your own needs!
#
# Tested successfully on:
# Ubuntu 10.04 Netbook Edition x86
# KUbuntu 11.04 x86
# openSuSE 11.4 x86
# XUbuntu 11.10 x64

source scripts/installFuncs.sh

if [ "$USER" == "root" ]; then
  echo "$0: ERROR this script should not been run as user root!"
  exit 10
fi

INSTALLPATH="400_lcd_320x240_olimex"
dir "$INSTALLPATH"
cd "$INSTALLPATH"

#{ IMPLEMENT YOUR INSTALL SCRIPT BELOW -----------------------------------------------

if [ ! -e OK.Documentation ]; then #{
  dir ../../Documentation/Boards
  getFile http://www.st.com/internet/com/TECHNICAL_RESOURCES/TECHNICAL_LITERATURE/DATASHEET/ CD00091417.pdf ../../Documentation/Boards/STM32-LCD_AccelerationSensor_LIS3LV02DL.pdf                                    || ERROR="1"
  
  if [ "$Error" == "" ]; then
    echo "Documentation complete: $INSTALLPATH."
    touch OK.Documentation
  fi
fi #}

if [ ! -e OK.Install ]; then #{
  echo "installing in $INSTALLPATH ..."
  InstallDir=`pwd`
  
  InstallInHomeDir="1"
  if [ "$InstallInHomeDir" == "1" ]; then
    Prefix=`pwd`  # install into subfolder of ToolChain/ for current user only
  else
    findPrefix    # install for all users
  fi
  
  Folder="DemoSoft"
  getFile http://olimex.com/dev/soft/arm/ST/ MOD-GSM%20and%20MOD-GSM-EDGE%20demo%20\(IAR%205\.50\).zip ${Folder}.zip 
  unZIP  $Folder ${Folder}.zip
  #add2CleanUp ../cleanup.sh "rm -Rf $INSTALLPATH/$Folder"
  
  if [ ! -d ${Folder} ]; then
    echo "ERROR: Cannot download file (folder ${Folder} not found in `pwd`)!"
    exit 5
  fi
  if [ -d ${Folder} ]; then
    cd "$Folder"
    
    File="modules/lcd-320x240.h"
    if [ ! -e "${File}_orig" ]; then #{
        makeOrig ${File}
        echo "`pwd`> modifing ${File}.. (avoids double definitions)"
    
        replaceInFile ${File} '#include "stm32f10x_lib.h"' '#include "stm32f10x.h"  //ToolChain_STM32 DISABLED #include "stm32f10x_lib.h"'
    fi #}
    
    File="modules/fonts.h"
    if [ ! -e "${File}_orig" ]; then #{
        makeOrig ${File}
        echo "`pwd`> modifing ${File}.. (avoids double definitions)"
    
        replaceInFile ${File} '#include "stm32f10x_lib.h"' '#include "stm32f10x.h"  //ToolChain_STM32 DISABLED #include "stm32f10x_lib.h"'
        cat <<END_OF_HEADER >temp.txt
#include "stm32f10x_gpio.h" //ADDED
#include "stm32f10x_fsmc.h" //ADDED
#include "stm32f10x_rcc.h"  //ADDED
#include "stm32_io.h"       //ADDED

END_OF_HEADER
        cat ${File} >>temp.txt
        mv temp.txt ${File}
    fi #}
  
    File="modules/lcd-320x240.c"
    if [ ! -e "${File}_orig" ]; then #{
        makeOrig ${File}
        echo "`pwd`> modifing ${File}.. (disabling float operations)"
    
        replaceInFile ${File} 'sqrt'  '1; //ToolChain_STM32 NO_FLOAT_SUPPORT  j=(int)sqrt(r\*r - i\*i);'
        replaceInFile ${File} 'asin'  '1; //ToolChain_STM32 NO_FLOAT_SUPPORT  polar_angle=asin((float)rel_py / polar_r);'
        replaceInFile ${File} 'floor' '1; //ToolChain_STM32 NO_FLOAT_SUPPORT  \*px=(int)floor(cos(polar_angle) \* polar_r) + x;'
        
    fi #}
    
    cd "$InstallDir"
    
    echo "" >OK.Install
  fi
fi #} [ ! -e OK.Install ]; then
if [   -e OK.Install ]; then #{
  Name="${INSTALLPATH}"
  createExtensionMakefileHead ${Name} #{
  cat <<END_OF_MAKEFILE >>../extensions/makefile.$Name 

# activate certain parts of source-code
COMPILE_OPTS += -DEXTENSION_${Name}=1

INCLUDE_DIRS += -I additionals/$Name/modules/
MAIN_OBJS += lcd-320x240.o sbrk.o

vpath %.c additionals/$Name/modules/

END_OF_MAKEFILE
  createExtensionMakefileTail ${Name} #}
  File="../extensions/activate.${Name}.sh" #{
  createActivateScriptHead $File $0 "low-level driver for LCD-panel provided on protoboard stm32_lcd from Olimex"
    cat <<END_OF_ACTIVATE >>$File #{

  # create links into extensions.active/
  createLink \$Dir_Extensions/makefile.$Name \$Dir_ExtensionsActive/makefile.$Name '' QUIET
  
  activate.250_stm_std_peripherals__fsmc.sh QUIET

END_OF_ACTIVATE
#}
  
  createActivateScriptTail $File $Name
  #}
  
  addLine ../scripts/createLinks.sh "rm 2>/dev/null $INSTALLPATH;           ln -sv \$Source/ToolChain_STM32/ToolChain/$INSTALLPATH/DemoSoft                                      $INSTALLPATH"
  echo "Installed successfully: $INSTALLPATH"
#}
else
  echo "failed to install $INSTALLPATH"
  exit 10
fi

cd ..

#} IMPLEMENT YOUR INSTALL SCRIPT ABOVE -----------------------------------------------

exit 0
