#!/usr/bin/perl

my $ObjectFiles = `ls *.o`;
my @ObjectFiles = split("\n", $ObjectFiles);

foreach my $ObjectFile (@ObjectFiles) {
  
  my $BSS  = `objdump -d -j .bss $ObjectFile`;
  my $Data = `objdump -d -j .data $ObjectFile`;
  my @BSS1  = split("\n", $BSS);  
  my @Data1 = split("\n", $Data);
  
  my @BSS;
  foreach my $BSS (@BSS1) {   # @BSS = all non empty entries from @BSS1
    my $Stripped = strip($BSS);
    if ($Stripped) { push @BSS, $Stripped; }
  }
  my @Data;
  foreach my $Data (@Data1) { # @Data = all non empty entries from @Data1
    my $Stripped = strip($Data);
    if ($Stripped) { push @Data, $Stripped; }
  }

  shift(@BSS); shift(@Data);  # remove first line containing filename 

  if (scalar(@BSS) + scalar(@Data) > 0) {
    my $TotalSize = 0;
    my @AllocationLines;
    
    my $CurrentAddress = 0;
    my $LastAddress    = 0;
    my %Regions = ( ' BSS' => \@BSS, DATA => \@Data);
    foreach my $RegionName (sort keys %Regions) {
      my $RegionRef = $Regions{$RegionName};
      if (@$RegionRef) {
        foreach my $Line (@$RegionRef) {
          if ( (substr($Line, 0, 1) eq '0') &&  # assume that first digit of address is always zero in our uC
               (substr($Line, 1, 1) ne ':')     # exclude '0:' lines
               ) {
            my $AddressHex = substr($Line, 0, index($Line, ' '));
            $CurrentAddress = hex($AddressHex);
            my $LastSize = $CurrentAddress - $LastAddress;
            $TotalSize += $LastSize;
            push @AllocationLines, $LastSize;
            
            my $PosLt = index($Line, '<');
            my $PosGt = index($Line, '>');
            my $SymbolName = substr($Line, $PosLt + 1, $PosGt - $PosLt - 1);
            push @AllocationLines, "  ".$RegionName.": ".$AddressHex.': '.$SymbolName.'';
          }
        }
      }
    }
    if (@AllocationLines) {
      print "\nFile: $ObjectFile allocates $TotalSize bytes:\n";
      shift(@AllocationLines); # first element is size 0
      while (@AllocationLines) {
        my $AllocationLine = shift(@AllocationLines);
        my $Size           = shift(@AllocationLines) + 0;
        print $AllocationLine." allocates $Size bytes\n";
      }
    }
  }
}

sub strip() {                  # removes trailing and leading whitechars from given string
  my $String = shift;
  
  unless (%s_WhiteChar) {
    $s_WhiteChar{"\n"} = 1;
    $s_WhiteChar{"\r"} = 1;
    $s_WhiteChar{"\t"} = 1;
    $s_WhiteChar{" "}  = 1;
  }
  while ( $s_WhiteChar{substr($String, 0, 1)} ) { substr($String, 0, 1) = undef; }
  while ( $s_WhiteChar{substr($String, -1, 1)} )   { substr($String, -1)   = undef; }
  
  return $String;
}
